@@ -8,8 +8,8 @@ from cognite.client._constants import DATA_MODELING_DEFAULT_LIMIT_READ
 from cognite.client.data_classes.data_modeling.containers import (
     Container,
     ContainerApply,
-    ContainerFilter,
     ContainerList,
+    _ContainerFilter,
 )
 from cognite.client.data_classes.data_modeling.ids import (
     ConstraintIdentifier,
@@ -72,14 +72,14 @@ class ContainersAPI(APIClient):
         Returns:
             Iterator[Container] | Iterator[ContainerList]: yields Container one by one if chunk_size is not specified, else ContainerList objects.
         """
-        filter = ContainerFilter(space, include_global)
+        flt = _ContainerFilter(space, include_global)
         return self._list_generator(
             list_cls=ContainerList,
             resource_cls=Container,
             method="GET",
             chunk_size=chunk_size,
             limit=limit,
-            filter=filter.dump(camel_case=True),
+            filter=flt.dump(camel_case=True),
         )
 
     def __iter__(self) -> Iterator[Container]:
@@ -113,12 +113,11 @@ class ContainersAPI(APIClient):
                 >>> client = CogniteClient()
                 >>> res = client.data_modeling.containers.retrieve(('mySpace', 'myContainer'))
 
-            Fetch using the ContainerId::
+            Fetch using the ContainerId:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import ContainerId
-                >>> client = CogniteClient()
-                >>> res = client.data_modeling.containers.retrieve(ContainerId(space='mySpace', external_id='myContainer'))
+                >>> res = client.data_modeling.containers.retrieve(
+                ...     ContainerId(space='mySpace', external_id='myContainer'))
         """
         identifier = _load_identifier(ids, "container")
         return self._retrieve_multiple(
@@ -137,7 +136,7 @@ class ContainersAPI(APIClient):
             list[ContainerId]: The container(s) which has been deleted. Empty list if nothing was deleted.
         Examples:
 
-            Delete containers by id::
+            Delete containers by id:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -163,7 +162,7 @@ class ContainersAPI(APIClient):
             list[ConstraintIdentifier]: The constraints(s) which have been deleted.
         Examples:
 
-            Delete constraints by id::
+            Delete constraints by id:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -182,7 +181,7 @@ class ContainersAPI(APIClient):
             list[IndexIdentifier]: The indexes(s) which has been deleted.
         Examples:
 
-            Delete indexes by id::
+            Delete indexes by id:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -242,28 +241,23 @@ class ContainersAPI(APIClient):
                 >>> client = CogniteClient()
                 >>> container_list = client.data_modeling.containers.list(limit=5)
 
-            Iterate over containers::
+            Iterate over containers:
 
-                >>> from cognite.client import CogniteClient
-                >>> client = CogniteClient()
                 >>> for container in client.data_modeling.containers:
                 ...     container # do something with the container
 
-            Iterate over chunks of containers to reduce memory load::
+            Iterate over chunks of containers to reduce memory load:
 
-                >>> from cognite.client import CogniteClient
-                >>> client = CogniteClient()
                 >>> for container_list in client.data_modeling.containers(chunk_size=10):
                 ...     container_list # do something with the containers
         """
-        filter = ContainerFilter(space, include_global)
-
+        flt = _ContainerFilter(space, include_global)
         return self._list(
             list_cls=ContainerList,
             resource_cls=Container,
             method="GET",
             limit=limit,
-            filter=filter.dump(camel_case=True),
+            filter=flt.dump(camel_case=True),
         )
 
     @overload
@@ -283,21 +277,29 @@ class ContainersAPI(APIClient):
 
         Examples:
 
-            Create new containers:
+            Create a new container:
 
                 >>> from cognite.client import CogniteClient
-                >>> from cognite.client.data_classes.data_modeling import ContainerApply, ContainerProperty, Text
+                >>> from cognite.client.data_classes.data_modeling import (
+                ...     ContainerApply, ContainerProperty, Text, Float64)
                 >>> client = CogniteClient()
-                >>> container = [ContainerApply(space="mySpace", external_id="myContainer",
-                ...     properties={"name": ContainerProperty(type=Text(), name="name")})]
+                >>> container = ContainerApply(
+                ...     space="mySpace",
+                ...     external_id="myContainer",
+                ...     properties={
+                ...         "name": ContainerProperty(type=Text, name="name"),
+                ...         "numbers": ContainerProperty(
+                ...             type=Float64(is_list=True, max_list_size=200),
+                ...             description="very important numbers",
+                ...         ),
+                ...     },
+                ... ),
                 >>> res = client.data_modeling.containers.apply(container)
 
             Create new container with unit-aware properties:
 
-                >>> from cognite.client import CogniteClient
-                >>> from cognite.client.data_classes.data_modeling import ContainerApply, ContainerProperty, Float64
+                >>> from cognite.client.data_classes.data_modeling import Float64
                 >>> from cognite.client.data_classes.data_modeling.data_types import UnitReference
-                >>> client = CogniteClient()
                 >>> container = ContainerApply(
                 ...     space="mySpace",
                 ...     external_id="myContainer",
@@ -327,6 +329,60 @@ class ContainersAPI(APIClient):
                 ...     }
                 ... )
                 >>> res = client.data_modeling.containers.apply(container)
+
+            Example container with all available properties (for illustration purposes). Note that
+            ``ContainerProperty`` has several options not shown here, like ``name``, ``description``,
+            ``nullable``, ``auto_increment``, ``default_value`` and ``immutable`` that may be specified,
+            depending on the choice of property type (e.g. ``auto_increment`` only works with integer types).
+
+                >>> from cognite.client.data_classes.data_modeling.data_types import UnitReference, EnumValue
+                >>> from cognite.client.data_classes.data_modeling.data_types import (
+                ...     Boolean, Date, DirectRelation, Enum, FileReference, Float32, Float64,
+                ...     Int32, Int64, Json, SequenceReference, Text, TimeSeriesReference, Timestamp
+                ... )
+                >>> container_properties = {
+                ...     "prop01": ContainerProperty(Boolean),
+                ...     "prop02": ContainerProperty(Boolean(is_list=True)),
+                ...     "prop03": ContainerProperty(Date),
+                ...     "prop04": ContainerProperty(Date(is_list=True)),
+                ...     "prop05": ContainerProperty(Timestamp),
+                ...     "prop06": ContainerProperty(Timestamp(is_list=True)),
+                ...     "prop07": ContainerProperty(Text),
+                ...     "prop08": ContainerProperty(Text(is_list=True)),
+                ...     # Note: DirectRelation(list) support `container`: The (optional) required type for the node
+                ...     #       the direct relation points to.
+                ...     "prop09": ContainerProperty(DirectRelation),
+                ...     "prop10": ContainerProperty(DirectRelation(is_list=True)),
+                ...     # Note: Enum also support `unknown_value`: The value to use when the enum value is unknown.
+                ...     "prop11": ContainerProperty(
+                ...         Enum({"Closed": EnumValue("Valve is closed"),
+                ...               "Opened": EnumValue("Valve is opened")})),
+                ...     # Note: Floats support unit references, e.g. `unit=UnitReference("pressure:bar")`:
+                ...     "prop12": ContainerProperty(Float32),
+                ...     "prop13": ContainerProperty(Float32(is_list=True)),
+                ...     "prop14": ContainerProperty(Float64),
+                ...     "prop15": ContainerProperty(Float64(is_list=True)),
+                ...     "prop16": ContainerProperty(Int32),
+                ...     "prop17": ContainerProperty(Int32(is_list=True)),
+                ...     "prop18": ContainerProperty(Int64),
+                ...     "prop19": ContainerProperty(Int64(is_list=True)),
+                ...     "prop20": ContainerProperty(Json),
+                ...     "prop21": ContainerProperty(Json(is_list=True)),
+                ...     "prop22": ContainerProperty(SequenceReference),
+                ...     "prop23": ContainerProperty(SequenceReference(is_list=True)),
+                ...     # Note: It is adviced to represent files and time series directly as nodes
+                ...     #       instead of referencing existing:
+                ...     "prop24": ContainerProperty(FileReference),
+                ...     "prop25": ContainerProperty(FileReference(is_list=True)),
+                ...     "prop26": ContainerProperty(TimeSeriesReference),
+                ...     "prop27": ContainerProperty(TimeSeriesReference(is_list=True)),
+                ... }
+                >>> container = ContainerApply(
+                ...     space="my-space",
+                ...     external_id="my-everything-container",
+                ...     properties=container_properties,
+                ... )
+
         """
         return self._create_multiple(
             list_cls=ContainerList,
