@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import inspect
+import itertools
 import logging
 import random
 import time
@@ -41,10 +42,14 @@ from cognite.client.data_classes.data_modeling.instances import (
     EdgeApplyResultList,
     EdgeList,
     InstanceAggregationResultList,
+    InstanceInspectResultList,
+    InstanceInspectResults,
     InstancesApplyResult,
     InstancesDeleteResult,
     InstanceSort,
     InstancesResult,
+    InvolvedContainers,
+    InvolvedViews,
     Node,
     NodeApply,
     NodeApplyResult,
@@ -55,17 +60,19 @@ from cognite.client.data_classes.data_modeling.instances import (
     T_Node,
     TargetUnit,
     TypedEdge,
+    TypedInstance,
     TypedNode,
     TypeInformation,
 )
 from cognite.client.data_classes.data_modeling.query import (
+    NodeOrEdgeResultSetExpression,
     Query,
     QueryResult,
     SourceSelector,
 )
 from cognite.client.data_classes.data_modeling.views import View
 from cognite.client.data_classes.filters import _BASIC_FILTERS, Filter, _validate_filter
-from cognite.client.utils._auxiliary import load_yaml_or_json
+from cognite.client.utils._auxiliary import is_unlimited, load_yaml_or_json
 from cognite.client.utils._concurrency import ConcurrencySettings
 from cognite.client.utils._identifier import DataModelingIdentifierSequence
 from cognite.client.utils._retry import Backoff
@@ -73,10 +80,10 @@ from cognite.client.utils._text import random_string
 from cognite.client.utils.useful_types import SequenceNotStr
 
 if TYPE_CHECKING:
-    from cognite.client import CogniteClient
+    from cognite.client import ClientConfig, CogniteClient
 
 _FILTERS_SUPPORTED: frozenset[type[Filter]] = _BASIC_FILTERS.union(
-    {filters.Nested, filters.HasData, filters.MatchAll, filters.Overlaps}
+    {filters.Nested, filters.HasData, filters.MatchAll, filters.Overlaps, filters.InstanceReferences}
 )
 
 logger = logging.getLogger(__name__)
@@ -98,7 +105,7 @@ class _NodeOrEdgeResourceAdapter(Generic[T_Node, T_Edge]):
 
 
 class _TypedNodeOrEdgeListAdapter:
-    def __init__(self, instance_cls: type) -> None:
+    def __init__(self, instance_cls: type[TypedInstance]) -> None:
         self._instance_cls = instance_cls
         self._list_cls = NodeList if issubclass(instance_cls, TypedNode) else EdgeList
 
@@ -158,6 +165,11 @@ class _NodeOrEdgeApplyAdapter:
 class InstancesAPI(APIClient):
     _RESOURCE_PATH = "/models/instances"
 
+    def __init__(self, config: ClientConfig, api_version: str | None, cognite_client: CogniteClient) -> None:
+        super().__init__(config, api_version, cognite_client)
+        self._AGGREGATE_LIMIT = 1000
+        self._SEARCH_LIMIT = 1000
+
     @overload
     def __call__(
         self,
@@ -525,9 +537,7 @@ class InstancesAPI(APIClient):
 
             Retrieve nodes an edges using the built in data class
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import NodeId, EdgeId, ViewId
-                >>> client = CogniteClient()
                 >>> res = client.data_modeling.instances.retrieve(
                 ...     NodeId("mySpace", "myNode"),
                 ...     EdgeId("mySpace", "myEdge"),
@@ -535,9 +545,7 @@ class InstancesAPI(APIClient):
 
             Retrieve nodes an edges using the the view object as source
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import NodeId, EdgeId
-                >>> client = CogniteClient()
                 >>> res = client.data_modeling.instances.retrieve(
                 ...     NodeId("mySpace", "myNode"),
                 ...     EdgeId("mySpace", "myEdge"),
@@ -677,16 +685,12 @@ class InstancesAPI(APIClient):
 
             Delete nodes and edges using the built in data class
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import NodeId, EdgeId
-                >>> client = CogniteClient()
                 >>> client.data_modeling.instances.delete(NodeId('mySpace', 'myNode'), EdgeId('mySpace', 'myEdge'))
 
             Delete all nodes from a NodeList
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import NodeId, EdgeId
-                >>> client = CogniteClient()
                 >>> my_view = client.data_modeling.views.retrieve(('mySpace', 'myView'))
                 >>> my_nodes = client.data_modeling.instances.list(instance_type='node', sources=my_view, limit=None)
                 >>> client.data_modeling.instances.delete(nodes=my_nodes.as_ids())
@@ -705,6 +709,73 @@ class InstancesAPI(APIClient):
         edge_ids = [EdgeId.load(item) for item in deleted_instances if item["instanceType"] == "edge"]
         return InstancesDeleteResult(node_ids, edge_ids)
 
+    def inspect(
+        self,
+        nodes: NodeId | Sequence[NodeId] | tuple[str, str] | Sequence[tuple[str, str]] | None = None,
+        edges: EdgeId | Sequence[EdgeId] | tuple[str, str] | Sequence[tuple[str, str]] | None = None,
+        *,
+        involved_views: InvolvedViews | None = None,
+        involved_containers: InvolvedContainers | None = None,
+    ) -> InstanceInspectResults:
+        """`Reverse lookup for instances. <https://developer.cognite.com/api/v1/#tag/Instances/operation/instanceInspect>`_
+
+        This method will return the involved views and containers for the given nodes and edges.
+
+        Args:
+            nodes (NodeId | Sequence[NodeId] | tuple[str, str] | Sequence[tuple[str, str]] | None): Node IDs.
+            edges (EdgeId | Sequence[EdgeId] | tuple[str, str] | Sequence[tuple[str, str]] | None): Edge IDs.
+            involved_views (InvolvedViews | None): Whether to include involved views. Must pass at least one of involved_views or involved_containers.
+            involved_containers (InvolvedContainers | None): Whether to include involved containers. Must pass at least one of involved_views or involved_containers.
+
+        Returns:
+            InstanceInspectResults: List of instance inspection results.
+
+        Examples:
+
+            Look up the involved views for a given node and edge:
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes.data_modeling import NodeId, EdgeId, InvolvedViews
+                >>> client = CogniteClient()
+                >>> res = client.data_modeling.instances.inspect(
+                ...     nodes=NodeId("my-space", "foo1"),
+                ...     edges=EdgeId("my-space", "bar2"),
+                ...     involved_views=InvolvedViews(all_versions=False),
+                ... )
+
+            Look up the involved containers:
+
+                >>> from cognite.client.data_classes.data_modeling import InvolvedContainers
+                >>> res = client.data_modeling.instances.inspect(
+                ...     nodes=[("my-space", "foo1"), ("my-space", "foo2")],
+                ...     involved_containers=InvolvedContainers(),
+                ... )
+        """
+        identifiers = self._load_node_and_edge_ids(nodes, edges)
+        inspect_operations = {}
+        # Only add the inspect operation to the dict if the value is provided. This is important
+        # since if it's omitted, the API will not execute the operation, making it cheaper.
+        if involved_views:
+            inspect_operations["involvedViews"] = {"allVersions": involved_views.all_versions}
+        if involved_containers:
+            inspect_operations["involvedContainers"] = {}
+        if not inspect_operations:
+            raise ValueError("Must pass at least one of 'involved_views' or 'involved_containers'")
+
+        items = list(
+            itertools.chain.from_iterable(
+                self._post(
+                    self._RESOURCE_PATH + "/inspect",
+                    json={"items": chunk.as_dicts(), "inspectionOperations": inspect_operations},
+                ).json()["items"]
+                for chunk in identifiers.chunked(1000)
+            )
+        )
+        return InstanceInspectResults(
+            nodes=InstanceInspectResultList._load([node for node in items if node["instanceType"] == "node"]),
+            edges=InstanceInspectResultList._load([edge for edge in items if edge["instanceType"] == "edge"]),
+        )
+
     def subscribe(
         self,
         query: Query,
@@ -739,16 +810,19 @@ class InstancesAPI(APIClient):
                 ...     print(result)
                 ...
                 >>> view_id = ViewId("someSpace", "someView", "v1")
-                >>> filter = Range(view_id.as_property_ref("releaseYear"), lt=2000)
+                >>> filter = Range(view_id.as_property_ref("createdYear"), lt=2023)
                 >>> query = Query(
-                ...     with_={"movies": NodeResultSetExpression(filter=filter)},
-                ...     select={"movies": Select([SourceSelector(view_id, ["releaseYear"])])}
+                ...     with_={"work_orders": NodeResultSetExpression(filter=filter)},
+                ...     select={"work_orders": Select([SourceSelector(view_id, ["createdYear"])])}
                 ... )
                 >>> subscription_context = client.data_modeling.instances.subscribe(query, just_print_the_result)
                 >>> subscription_context.cancel()
         """
         for result_set_expression in query.with_.values():
-            if result_set_expression.from_ is not None:
+            if (
+                isinstance(result_set_expression, NodeOrEdgeResultSetExpression)
+                and result_set_expression.from_ is not None
+            ):
                 raise ValueError("Cannot chain result sets when subscribing to a query")
 
         subscription_context = SubscriptionContext()
@@ -810,9 +884,10 @@ class InstancesAPI(APIClient):
         other_params: dict[str, Any] = {"includeTyping": include_typing}
         if sources:
             other_params["sources"] = [source.dump() for source in SourceSelector._load_list(sources)]
-            if with_properties := [s["viewId"] for s in other_params["sources"] if "properties" in s]:
+            if with_properties := [s["source"] for s in other_params["sources"] if "properties" in s]:
                 raise ValueError(
-                    f"Properties are not supported in this context. Got in source argument for view IDs {with_properties}"
+                    f"Selecting properties is not supported in this context. "
+                    f"Received in `sources` argument for views: {with_properties}."
                 )
         if sort:
             if isinstance(sort, (InstanceSort, dict)):
@@ -862,56 +937,49 @@ class InstancesAPI(APIClient):
 
             Create two nodes with data with a one-to-many edge
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import EdgeApply, NodeOrEdgeData, NodeApply, ViewId
-                >>> actor = NodeApply(
-                ...     space="actors",
-                ...     external_id="arnold_schwarzenegger",
+                >>> work_order = NodeApply(
+                ...     space="industrial",
+                ...     external_id="work_order:123",
                 ...     sources=[
                 ...         NodeOrEdgeData(
-                ...             ViewId("mySpace", "PersonView", "v1"),
-                ...             {"name": "Arnold Schwarzenegger", "birthYear": 1947}
-                ...         ),
-                ...         NodeOrEdgeData(
-                ...             ViewId("mySpace", "ActorView", "v1"),
-                ...             {"wonOscar": False}
+                ...             ViewId("mySpace", "WorkOrderView", "v1"),
+                ...             {"title": "Repair pump", "createdYear": 2023}
                 ...         )
                 ...     ]
                 ... )
-                >>> movie = NodeApply(
-                ...     space="movies",
-                ...     external_id="Terminator",
+                >>> pump = NodeApply(
+                ...     space="industrial",
+                ...     external_id="pump:456",
                 ...     sources=[
                 ...         NodeOrEdgeData(
-                ...             ViewId("mySpace", "MovieView", "v1"),
-                ...             {"title": "Terminator", "releaseYear": 1984}
+                ...             ViewId("mySpace", "PumpView", "v1"),
+                ...             {"name": "Pump 456", "location": "Subsea"}
                 ...         )
                 ...     ]
                 ... )
-                ... # This is one-to-many edge, in this case from a person to a movie
-                >>> actor_to_movie = EdgeApply(
-                ...     space="actors",
-                ...     external_id="relation:arnold_schwarzenegger:terminator",
-                ...     type=("types", "acts-in"),
-                ...     start_node=("actors", "arnold_schwarzenegger"),
-                ...     end_node=("movies", "Terminator"),
+                ... # This is one-to-many edge, in this case from a work order to a pump
+                >>> work_order_to_pump = EdgeApply(
+                ...     space="industrial",
+                ...     external_id="relation:work_order:123:pump:456",
+                ...     type=("industrial", "relates-to"),
+                ...     start_node=("industrial", "work_order:123"),
+                ...     end_node=("industrial", "pump:456"),
                 ... )
-                >>> res = client.data_modeling.instances.apply([actor, movie], [actor_to_movie])
+                >>> res = client.data_modeling.instances.apply([work_order, pump], [work_order_to_pump])
 
             Create new edge and automatically create end nodes.
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import EdgeApply
-                >>> client = CogniteClient()
-                >>> actor_to_movie = EdgeApply(
-                ...     space="actors",
-                ...     external_id="relation:arnold_schwarzenegger:terminator",
-                ...     type=("types", "acts-in"),
-                ...     start_node=("actors", "arnold_schwarzenegger"),
-                ...     end_node=("movies", "Terminator"),
+                >>> work_order_to_pump = EdgeApply(
+                ...     space="industrial",
+                ...     external_id="relation:work_order:123:pump:456",
+                ...     type=("industrial", "relates-to"),
+                ...     start_node=("industrial", "work_order:123"),
+                ...     end_node=("industrial", "pump:456"),
                 ... )
                 >>> res = client.data_modeling.instances.apply(
-                ...     edges=actor_to_movie,
+                ...     edges=work_order_to_pump,
                 ...     auto_create_start_nodes=True,
                 ...     auto_create_end_nodes=True
                 ... )
@@ -927,7 +995,6 @@ class InstancesAPI(APIClient):
             `my_node.birth_year` to return the data for property `birthYear`, must use the PropertyOptions as shown below.
             We strongly suggest you use snake_cased attribute names, as is done here:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import TypedNodeApply, PropertyOptions
                 >>> class PersonApply(TypedNodeApply):
                 ...     birth_year = PropertyOptions(identifier="birthYear")
@@ -980,7 +1047,7 @@ class InstancesAPI(APIClient):
         space: str | SequenceNotStr[str] | None = None,
         filter: Filter | dict[str, Any] | None = None,
         include_typing: bool = False,
-        limit: int = DEFAULT_LIMIT_READ,
+        limit: int | None = DEFAULT_LIMIT_READ,
         sort: Sequence[InstanceSort | dict] | InstanceSort | dict | None = None,
     ) -> NodeList[Node]: ...
 
@@ -996,7 +1063,7 @@ class InstancesAPI(APIClient):
         space: str | SequenceNotStr[str] | None = None,
         filter: Filter | dict[str, Any] | None = None,
         include_typing: bool = False,
-        limit: int = DEFAULT_LIMIT_READ,
+        limit: int | None = DEFAULT_LIMIT_READ,
         sort: Sequence[InstanceSort | dict] | InstanceSort | dict | None = None,
     ) -> EdgeList[Edge]: ...
 
@@ -1012,7 +1079,7 @@ class InstancesAPI(APIClient):
         space: str | SequenceNotStr[str] | None = None,
         filter: Filter | dict[str, Any] | None = None,
         include_typing: bool = False,
-        limit: int = DEFAULT_LIMIT_READ,
+        limit: int | None = DEFAULT_LIMIT_READ,
         sort: Sequence[InstanceSort | dict] | InstanceSort | dict | None = None,
     ) -> NodeList[T_Node]: ...
 
@@ -1028,7 +1095,7 @@ class InstancesAPI(APIClient):
         space: str | SequenceNotStr[str] | None = None,
         filter: Filter | dict[str, Any] | None = None,
         include_typing: bool = False,
-        limit: int = DEFAULT_LIMIT_READ,
+        limit: int | None = DEFAULT_LIMIT_READ,
         sort: Sequence[InstanceSort | dict] | InstanceSort | dict | None = None,
     ) -> EdgeList[T_Edge]: ...
 
@@ -1042,7 +1109,7 @@ class InstancesAPI(APIClient):
         space: str | SequenceNotStr[str] | None = None,
         filter: Filter | dict[str, Any] | None = None,
         include_typing: bool = False,
-        limit: int = DEFAULT_LIMIT_READ,
+        limit: int | None = DEFAULT_LIMIT_READ,
         sort: Sequence[InstanceSort | dict] | InstanceSort | dict | None = None,
     ) -> NodeList[T_Node] | EdgeList[T_Edge]:
         """`Search instances <https://developer.cognite.com/api/v1/#tag/Instances/operation/searchInstances>`_
@@ -1056,7 +1123,8 @@ class InstancesAPI(APIClient):
             space (str | SequenceNotStr[str] | None): Restrict instance search to the given space (or list of spaces).
             filter (Filter | dict[str, Any] | None): Advanced filtering of instances.
             include_typing (bool): Whether to include typing information.
-            limit (int): Maximum number of instances to return. Defaults to 25.
+            limit (int | None): Maximum number of instances to return. Defaults to 25. Will return the maximum number
+                of results (1000) if set to None, -1, or math.inf.
             sort (Sequence[InstanceSort | dict] | InstanceSort | dict | None): How you want the listed instances information ordered.
 
         Returns:
@@ -1076,10 +1144,8 @@ class InstancesAPI(APIClient):
 
             Search for Quentin in the person view in the name property, but only born after 1970:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import ViewId
                 >>> from cognite.client.data_classes import filters
-                >>> client = CogniteClient()
                 >>> born_after_1970 = filters.Range(["mySpace", "PersonView/v1", "birthYear"], gt=1970)
                 >>> res = client.data_modeling.instances.search(
                 ...     ViewId("mySpace", "PersonView", "v1"),
@@ -1106,7 +1172,11 @@ class InstancesAPI(APIClient):
         else:
             raise ValueError(f"Invalid instance type: {instance_type}")
 
-        body: dict[str, Any] = {"view": view.dump(camel_case=True), "instanceType": instance_type_str, "limit": limit}
+        body: dict[str, Any] = {
+            "view": view.dump(camel_case=True),
+            "instanceType": instance_type_str,
+            "limit": self._SEARCH_LIMIT if is_unlimited(limit) else limit,
+        }
         if query:
             body["query"] = query
         if properties:
@@ -1142,7 +1212,7 @@ class InstancesAPI(APIClient):
         target_units: list[TargetUnit] | None = None,
         space: str | SequenceNotStr[str] | None = None,
         filter: Filter | dict[str, Any] | None = None,
-        limit: int = DEFAULT_LIMIT_READ,
+        limit: int | None = DEFAULT_LIMIT_READ,
     ) -> AggregatedNumberedValue: ...
 
     @overload
@@ -1157,7 +1227,7 @@ class InstancesAPI(APIClient):
         target_units: list[TargetUnit] | None = None,
         space: str | SequenceNotStr[str] | None = None,
         filter: Filter | dict[str, Any] | None = None,
-        limit: int = DEFAULT_LIMIT_READ,
+        limit: int | None = DEFAULT_LIMIT_READ,
     ) -> list[AggregatedNumberedValue]: ...
 
     @overload
@@ -1172,7 +1242,7 @@ class InstancesAPI(APIClient):
         target_units: list[TargetUnit] | None = None,
         space: str | SequenceNotStr[str] | None = None,
         filter: Filter | dict[str, Any] | None = None,
-        limit: int = DEFAULT_LIMIT_READ,
+        limit: int | None = DEFAULT_LIMIT_READ,
     ) -> InstanceAggregationResultList: ...
 
     def aggregate(
@@ -1186,7 +1256,7 @@ class InstancesAPI(APIClient):
         target_units: list[TargetUnit] | None = None,
         space: str | SequenceNotStr[str] | None = None,
         filter: Filter | dict[str, Any] | None = None,
-        limit: int = DEFAULT_LIMIT_READ,
+        limit: int | None = DEFAULT_LIMIT_READ,
     ) -> AggregatedNumberedValue | list[AggregatedNumberedValue] | InstanceAggregationResultList:
         """`Aggregate data across nodes/edges <https://developer.cognite.com/api/v1/#tag/Instances/operation/aggregateInstances>`_
 
@@ -1200,20 +1270,21 @@ class InstancesAPI(APIClient):
             target_units (list[TargetUnit] | None): Properties to convert to another unit. The API can only convert to another unit if a unit has been defined as part of the type on the underlying container being queried.
             space (str | SequenceNotStr[str] | None): Restrict instance aggregate query to the given space (or list of spaces).
             filter (Filter | dict[str, Any] | None): Advanced filtering of instances.
-            limit (int): Maximum number of instances to return. Defaults to 25.
+            limit (int | None): Maximum number of instances to return. Defaults to 25. Will return the maximum number
+                of results (1000) if set to None, -1, or math.inf.
 
         Returns:
             AggregatedNumberedValue | list[AggregatedNumberedValue] | InstanceAggregationResultList: Node or edge aggregation results.
 
         Examples:
 
-            Get the average run time in minutes for movies grouped by release year:
+            Get the average run time in minutes for pumps grouped by release year:
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling import ViewId, aggregations as aggs
                 >>> client = CogniteClient()
                 >>> avg_run_time = aggs.Avg("runTimeMinutes")
-                >>> view_id = ViewId("mySpace", "PersonView", "v1")
+                >>> view_id = ViewId("mySpace", "PumpView", "v1")
                 >>> res = client.data_modeling.instances.aggregate(view_id, avg_run_time, group_by="releaseYear")
 
         """
@@ -1222,7 +1293,11 @@ class InstancesAPI(APIClient):
 
         self._validate_filter(filter)
         filter = self._merge_space_into_filter(instance_type, space, filter)
-        body: dict[str, Any] = {"view": view.dump(camel_case=True), "instanceType": instance_type, "limit": limit}
+        body: dict[str, Any] = {
+            "view": view.dump(camel_case=True),
+            "instanceType": instance_type,
+            "limit": self._AGGREGATE_LIMIT if is_unlimited(limit) else limit,
+        }
         is_single = isinstance(aggregates, (dict, MetricAggregation))
         aggregate_seq: Sequence[MetricAggregation | dict] = (
             [aggregates] if isinstance(aggregates, (dict, MetricAggregation)) else aggregates
@@ -1369,27 +1444,47 @@ class InstancesAPI(APIClient):
 
         Examples:
 
-            Find actors in movies released before 2000 sorted by actor name:
+            Find work orders created before 2023 sorted by title:
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling.query import Query, Select, NodeResultSetExpression, EdgeResultSetExpression, SourceSelector
                 >>> from cognite.client.data_classes.filters import Range, Equals
                 >>> from cognite.client.data_classes.data_modeling.ids import ViewId
                 >>> client = CogniteClient()
-                >>> movie_id = ViewId("mySpace", "MovieView", "v1")
-                >>> actor_id = ViewId("mySpace", "ActorView", "v1")
+                >>> work_order_id = ViewId("mySpace", "WorkOrderView", "v1")
+                >>> pump_id = ViewId("mySpace", "PumpView", "v1")
                 >>> query = Query(
                 ...     with_ = {
-                ...         "movies": NodeResultSetExpression(filter=Range(movie_id.as_property_ref("releaseYear"), lt=2000)),
-                ...         "actors_in_movie": EdgeResultSetExpression(from_="movies", filter=Equals(["edge", "type"], {"space": movie_id.space, "externalId": "Movie.actors"})),
-                ...         "actors": NodeResultSetExpression(from_="actors_in_movie"),
+                ...         "work_orders": NodeResultSetExpression(filter=Range(work_order_id.as_property_ref("createdYear"), lt=2023)),
+                ...         "work_orders_to_pumps": EdgeResultSetExpression(from_="work_orders", filter=Equals(["edge", "type"], {"space": work_order_id.space, "externalId": "WorkOrder.asset"})),
+                ...         "pumps": NodeResultSetExpression(from_="work_orders_to_pumps"),
                 ...     },
                 ...     select = {
-                ...         "actors": Select(
-                ...             [SourceSelector(actor_id, ["name"])], sort=[InstanceSort(actor_id.as_property_ref("name"))]),
+                ...         "pumps": Select(
+                ...             [SourceSelector(pump_id, ["name"])], sort=[InstanceSort(pump_id.as_property_ref("name"))]),
                 ...     },
                 ... )
                 >>> res = client.data_modeling.instances.query(query)
+
+            To convert units, specify what your target units are in the SourceSelector. You can either use
+            a UnitReference or a UnitSystemReference. Note that in order for a property to be converted, they
+            need to have a unit defined in the underlying container.
+
+                >>> from cognite.client.data_classes.data_modeling.data_types import UnitReference, UnitSystemReference
+                >>> selected_source = SourceSelector(
+                ...     source=ViewId("my-space", "my-xid", "v1"),
+                ...     properties=["f32_prop1", "f32_prop2", "f64_prop1", "f64_prop2"],
+                ...     target_units=[
+                ...         TargetUnit("f32_prop1", UnitReference("pressure:kilopa")),
+                ...         TargetUnit("f32_prop2", UnitReference("pressure:barg")),
+                ...         TargetUnit("f64_prop1", UnitSystemReference("SI")),
+                ...         TargetUnit("f64_prop2", UnitSystemReference("Imperial")),
+                ...     ],
+                ... )
+
+            To select all properties, use '[*]' in your SourceSelector:
+
+                >>> SourceSelector(source=ViewId("my-space", "my-xid", "v1"), properties=["*"])
         """
         return self._query_or_sync(query, "query", include_typing)
 
@@ -1407,7 +1502,7 @@ class InstancesAPI(APIClient):
 
         Examples:
 
-            Find actors in movies released before 2000 sorted by actor name:
+            Find work orders created before 2023 sorted by title:
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes.data_modeling.instances import InstanceSort
@@ -1415,25 +1510,25 @@ class InstancesAPI(APIClient):
                 >>> from cognite.client.data_classes.filters import Range, Equals
                 >>> from cognite.client.data_classes.data_modeling.ids import ViewId
                 >>> client = CogniteClient()
-                >>> movie_id = ViewId("mySpace", "MovieView", "v1")
-                >>> actor_id = ViewId("mySpace", "ActorView", "v1")
+                >>> work_order_id = ViewId("mySpace", "WorkOrderView", "v1")
+                >>> pump_id = ViewId("mySpace", "PumpView", "v1")
                 >>> query = Query(
                 ...     with_ = {
-                ...         "movies": NodeResultSetExpression(filter=Range(movie_id.as_property_ref("releaseYear"), lt=2000)),
-                ...         "actors_in_movie": EdgeResultSetExpression(from_="movies", filter=Equals(["edge", "type"], {"space": movie_id.space, "externalId": "Movie.actors"})),
-                ...         "actors": NodeResultSetExpression(from_="actors_in_movie"),
+                ...         "work_orders": NodeResultSetExpression(filter=Range(work_order_id.as_property_ref("createdYear"), lt=2023)),
+                ...         "work_orders_to_pumps": EdgeResultSetExpression(from_="work_orders", filter=Equals(["edge", "type"], {"space": work_order_id.space, "externalId": "WorkOrder.asset"})),
+                ...         "pumps": NodeResultSetExpression(from_="work_orders_to_pumps"),
                 ...     },
                 ...     select = {
-                ...         "actors": Select(
-                ...             [SourceSelector(actor_id, ["name"])], sort=[InstanceSort(actor_id.as_property_ref("name"))]),
+                ...         "pumps": Select(
+                ...             [SourceSelector(pump_id, ["name"])], sort=[InstanceSort(pump_id.as_property_ref("name"))]),
                 ...     },
                 ... )
                 >>> res = client.data_modeling.instances.sync(query)
-                >>> # Added a new movie with actors released before 2000
+                >>> # Added a new work order with pumps created before 2023
                 >>> query.cursors = res.cursors
                 >>> res_new = client.data_modeling.instances.sync(query)
 
-            In the last example, the res_new will only contain the actors that have been added with the new movie.
+            In the last example, the res_new will only contain the pumps that have been added with the new work order.
         """
         return self._query_or_sync(query, "sync", include_typing=include_typing)
 
@@ -1569,10 +1664,10 @@ class InstancesAPI(APIClient):
         elif instance_type == "edge":
             resource_cls, list_cls = Edge, EdgeList
         elif inspect.isclass(instance_type) and issubclass(instance_type, TypedNode):
-            resource_cls = _NodeOrEdgeResourceAdapter(instance_type, Edge)  # type: ignore[assignment]
+            resource_cls = instance_type
             list_cls = _TypedNodeOrEdgeListAdapter(instance_type)  # type: ignore[assignment]
         elif inspect.isclass(instance_type) and issubclass(instance_type, TypedEdge):
-            resource_cls = _NodeOrEdgeResourceAdapter(Node, instance_type)  # type: ignore[assignment]
+            resource_cls = instance_type
             list_cls = _TypedNodeOrEdgeListAdapter(instance_type)  # type: ignore[assignment]
         else:
             raise ValueError(f"Invalid instance type: {instance_type}")
@@ -1605,7 +1700,9 @@ class InstancesAPI(APIClient):
         space_filter = filters.SpaceFilter(space, instance_type)
         if filter is None:
             return space_filter
-        return filters.And(space_filter, Filter.load(filter) if isinstance(filter, dict) else filter)
+        filter = Filter.load(filter) if isinstance(filter, dict) else filter
+        # 'And' the space filter with the user filter (will merge if user filter is 'And')
+        return space_filter & filter
 
     @staticmethod
     def _to_instance_type_str(
