@@ -85,13 +85,23 @@ class DatapointsSubscriptionAPI(APIClient):
                 ...     time_series_ids=["myFistTimeSeries", "mySecondTimeSeries"])
                 >>> created = client.time_series.subscriptions.create(sub)
 
+            Create a subscription with explicit time series IDs given as Node IDs
+            either from CogniteTimeSeries or an extension of CogniteTimeseries:
+
+                >>> from cognite.client.data_classes import DataPointSubscriptionWrite
+                >>> from cognite.client.data_classes.data_modeling import NodeId
+                >>> sub = DataPointSubscriptionWrite(
+                ...     external_id="my_subscription",
+                ...     name="My subscription with Data Model Ids",
+                ...     partition_count=1,
+                ...     instance_ids=[NodeId("my_space", "myFistTimeSeries"), NodeId("my_space", "mySecondTimeSeries")])
+                >>> created = client.time_series.subscriptions.create(sub)
+
             Create a filter defined subscription for all numeric time series that are stepwise:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import DataPointSubscriptionWrite
                 >>> from cognite.client.data_classes import filters as flt
                 >>> from cognite.client.data_classes.datapoints_subscriptions import DatapointSubscriptionProperty
-                >>> client = CogniteClient()
                 >>> is_numeric_stepwise = flt.And(
                 ...     flt.Equals(DatapointSubscriptionProperty.is_string, False),
                 ...     flt.Equals(DatapointSubscriptionProperty.is_step, True))
@@ -223,9 +233,7 @@ class DatapointsSubscriptionAPI(APIClient):
 
             Add a time series to a preexisting subscription:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import DataPointSubscriptionUpdate
-                >>> client = CogniteClient()
                 >>> update = DataPointSubscriptionUpdate("my_subscription").time_series_ids.add(["MyNewTimeSeriesExternalId"])
                 >>> updated = client.time_series.subscriptions.update(update)
         """
