@@ -30,7 +30,6 @@ from cognite.client.data_classes.data_modeling import NodeId
 from cognite.client.exceptions import CogniteAPIError, CogniteAuthorizationError, CogniteFileUploadError
 from cognite.client.utils._auxiliary import find_duplicates
 from cognite.client.utils._concurrency import execute_tasks
-from cognite.client.utils._experimental import FeaturePreviewWarning
 from cognite.client.utils._identifier import Identifier, IdentifierSequence
 from cognite.client.utils._validation import process_asset_subtree_ids, process_data_set_ids
 from cognite.client.utils.useful_types import SequenceNotStr
@@ -39,10 +38,6 @@ from cognite.client.utils.useful_types import SequenceNotStr
 class FilesAPI(APIClient):
     _RESOURCE_PATH = "/files"
 
-    @staticmethod
-    def _warn_alpha() -> None:
-        FeaturePreviewWarning(api_maturity="alpha", feature_name="Files with Instance ID", sdk_maturity="alpha").warn()
-
     @overload
     def __call__(
         self,
@@ -213,7 +208,7 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Create a file::
+            Create a file:
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import FileMetadataWrite
@@ -247,26 +242,18 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Get file metadata by id::
+            Get file metadata by id:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
                 >>> res = client.files.retrieve(id=1)
 
-            Get file metadata by external id::
+            Get file metadata by external id:
 
-                >>> from cognite.client import CogniteClient
-                >>> client = CogniteClient()
                 >>> res = client.files.retrieve(external_id="1")
         """
-        headers: dict | None = None
-        if instance_id is not None:
-            self._warn_alpha()
-            headers = {"cdf-version": "alpha"}
         identifiers = IdentifierSequence.load(ids=id, external_ids=external_id, instance_ids=instance_id).as_singleton()
-        return self._retrieve_multiple(
-            list_cls=FileMetadataList, resource_cls=FileMetadata, identifiers=identifiers, headers=headers
-        )
+        return self._retrieve_multiple(list_cls=FileMetadataList, resource_cls=FileMetadata, identifiers=identifiers)
 
     def retrieve_multiple(
         self,
@@ -288,29 +275,22 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Get file metadatas by id::
+            Get file metadatas by id:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
                 >>> res = client.files.retrieve_multiple(ids=[1, 2, 3])
 
-            Get file_metadatas by external id::
+            Get file_metadatas by external id:
 
-                >>> from cognite.client import CogniteClient
-                >>> client = CogniteClient()
                 >>> res = client.files.retrieve_multiple(external_ids=["abc", "def"])
         """
-        header: dict | None = None
-        if instance_ids is not None:
-            self._warn_alpha()
-            header = {"cdf-version": "alpha"}
         identifiers = IdentifierSequence.load(ids=ids, external_ids=external_ids, instance_ids=instance_ids)
         return self._retrieve_multiple(
             list_cls=FileMetadataList,
             resource_cls=FileMetadata,
             identifiers=identifiers,
             ignore_unknown_ids=ignore_unknown_ids,
-            headers=header,
         )
 
     def aggregate(self, filter: FileMetadataFilter | dict[str, Any] | None = None) -> list[CountAggregate]:
@@ -324,7 +304,7 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            List files metadata and filter on external id prefix::
+            List files metadata and filter on external id prefix:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -348,7 +328,7 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Delete files by id or external id::
+            Delete files by id or external id:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -394,7 +374,7 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Update file metadata that you have fetched. This will perform a full update of the file metadata::
+            Update file metadata that you have fetched. This will perform a full update of the file metadata:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -402,44 +382,30 @@ class FilesAPI(APIClient):
                 >>> file_metadata.description = "New description"
                 >>> res = client.files.update(file_metadata)
 
-            Perform a partial update on file metadata, updating the source and adding a new field to metadata::
+            Perform a partial update on file metadata, updating the source and adding a new field to metadata:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import FileMetadataUpdate
-                >>> client = CogniteClient()
                 >>> my_update = FileMetadataUpdate(id=1).source.set("new source").metadata.add({"key": "value"})
                 >>> res = client.files.update(my_update)
 
-            Attach labels to a files::
+            Attach labels to a files:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import FileMetadataUpdate
-                >>> client = CogniteClient()
                 >>> my_update = FileMetadataUpdate(id=1).labels.add(["PUMP", "VERIFIED"])
                 >>> res = client.files.update(my_update)
 
-            Detach a single label from a file::
+            Detach a single label from a file:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import FileMetadataUpdate
-                >>> client = CogniteClient()
                 >>> my_update = FileMetadataUpdate(id=1).labels.remove("PUMP")
                 >>> res = client.files.update(my_update)
         """
-        headers: dict | None = None
-        if (isinstance(item, Sequence) and any(file.instance_id for file in item)) or (
-            not isinstance(item, Sequence) and item.instance_id
-        ):
-            self._warn_alpha()
-            headers = {"cdf-version": "alpha"}
-
         return self._update_multiple(
             list_cls=FileMetadataList,
             resource_cls=FileMetadata,
             update_cls=FileMetadataUpdate,
             resource_path=self._RESOURCE_PATH,
             items=item,
-            headers=headers,
             mode=mode,
         )
 
@@ -462,7 +428,7 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Search for a file::
+            Search for a file:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -470,8 +436,6 @@ class FilesAPI(APIClient):
 
             Search for an asset with an attached label:
 
-                >>> from cognite.client import CogniteClient
-                >>> client = CogniteClient()
                 >>> my_label_filter = LabelFilter(contains_all=["WELL LOG"])
                 >>> res = client.assets.search(name="xyz",filter=FileMetadataFilter(labels=my_label_filter))
         """
@@ -547,7 +511,7 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Upload a file in a given path::
+            Upload a file in a given path:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -555,28 +519,20 @@ class FilesAPI(APIClient):
 
             If name is omitted, this method will use the name of the file
 
-                >>> from cognite.client import CogniteClient
-                >>> client = CogniteClient()
                 >>> res = client.files.upload("/path/to/file")
 
-            You can also upload all files in a directory by setting path to the path of a directory::
+            You can also upload all files in a directory by setting path to the path of a directory:
 
-                >>> from cognite.client import CogniteClient
-                >>> client = CogniteClient()
                 >>> res = client.files.upload("/path/to/my/directory")
 
-            Upload a file with a label::
+            Upload a file with a label:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import Label
-                >>> client = CogniteClient()
                 >>> res = client.files.upload("/path/to/file", name="my_file", labels=[Label(external_id="WELL LOG")])
 
-            Upload a file with a geo_location::
+            Upload a file with a geo_location:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import GeoLocation, Geometry
-                >>> client = CogniteClient()
                 >>> geometry = Geometry(type="LineString", coordinates=[[30, 10], [10, 30], [40, 40]])
                 >>> res = client.files.upload("/path/to/file", geo_location=GeoLocation(type="Feature", geometry=geometry))
 
@@ -633,16 +589,16 @@ class FilesAPI(APIClient):
 
     def upload_content_bytes(
         self,
-        content: str | bytes | TextIO | BinaryIO,
+        content: str | bytes | BinaryIO,
         external_id: str | None = None,
         instance_id: NodeId | None = None,
     ) -> FileMetadata:
-        """Upload bytes or string.
+        """Upload bytes or string (UTF-8 assumed).
 
         Note that the maximum file size is 5GiB. In order to upload larger files use `multipart_upload_content_session`.
 
         Args:
-            content (str | bytes | TextIO | BinaryIO): The content to upload.
+            content (str | bytes | BinaryIO): The content to upload.
             external_id (str | None): The external ID provided by the client. Must be unique within the project.
             instance_id (NodeId | None): Instance ID of the file.
 
@@ -651,35 +607,32 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Upload a file from memory::
+            Finish a file creation by uploading the content using external_id:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
-                >>> res = client.files.upload_bytes(b"some content", name="my_file", asset_ids=[1,2,3])
+                >>> res = client.files.upload_content_bytes(
+                ...     b"some content", external_id="my_file_xid")
+
+            ...or by using instance_id:
+
+                >>> from cognite.client.data_classes.data_modeling import NodeId
+                >>> res = client.files.upload_content_bytes(
+                ...     b"some content", instance_id=NodeId("my-space", "my_file_xid"))
         """
-        headers: dict | None = None
-        if instance_id is not None:
-            self._warn_alpha()
-            headers = {"cdf-version": "alpha"}
         identifiers = IdentifierSequence.load(external_ids=external_id, instance_ids=instance_id).as_singleton()
 
         if isinstance(content, str):
             content = content.encode("utf-8")
 
         try:
-            res = self._post(
-                url_path=f"{self._RESOURCE_PATH}/uploadlink",
-                json={"items": identifiers.as_dicts()},
-                headers=headers,
-            )
+            res = self._post(url_path=f"{self._RESOURCE_PATH}/uploadlink", json={"items": identifiers.as_dicts()})
         except CogniteAPIError as e:
             if e.code == 403:
                 raise CogniteAuthorizationError(message=e.message, code=e.code, x_request_id=e.x_request_id) from e
             raise
 
-        file_metadata = self._upload_bytes(content, res.json()["items"][0])
-
-        return file_metadata
+        return self._upload_bytes(content, res.json()["items"][0])
 
     def _upload_bytes(self, content: bytes | TextIO | BinaryIO, returned_file_metadata: dict) -> FileMetadata:
         upload_url = returned_file_metadata["uploadUrl"]
@@ -687,26 +640,21 @@ class FilesAPI(APIClient):
             full_upload_url = upload_url
         else:
             full_upload_url = urljoin(self._config.base_url, upload_url)
-        file_metadata = FileMetadata.load(returned_file_metadata)
-        headers = {"Content-Type": file_metadata.mime_type}
+        file_metadata = FileMetadata._load(returned_file_metadata)
         upload_response = self._http_client_with_retry.request(
             "PUT",
             full_upload_url,
-            accept="*/*",
             data=content,
             timeout=self._config.file_transfer_timeout,
-            headers=headers,
+            headers={"Content-Type": file_metadata.mime_type, "accept": "*/*"},
         )
         if not upload_response.ok:
-            raise CogniteFileUploadError(
-                message=upload_response.text,
-                code=upload_response.status_code,
-            )
+            raise CogniteFileUploadError(message=upload_response.text, code=upload_response.status_code)
         return file_metadata
 
     def upload_bytes(
         self,
-        content: str | bytes | TextIO | BinaryIO,
+        content: str | bytes | BinaryIO,
         name: str,
         external_id: str | None = None,
         source: str | None = None,
@@ -724,12 +672,12 @@ class FilesAPI(APIClient):
     ) -> FileMetadata:
         """Upload bytes or string.
 
-        You can also pass a file handle to content.
+        You can also pass a file handle to 'content'. The file should be opened in binary mode.
 
         Note that the maximum file size is 5GiB. In order to upload larger files use `multipart_upload_session`.
 
         Args:
-            content (str | bytes | TextIO | BinaryIO): The content to upload.
+            content (str | bytes | BinaryIO): The content to upload.
             name (str): Name of the file.
             external_id (str | None): The external ID provided by the client. Must be unique within the project.
             source (str | None): The source of the file.
@@ -750,7 +698,7 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Upload a file from memory::
+            Upload a file from memory:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -812,7 +760,7 @@ class FilesAPI(APIClient):
         The file chunks may be uploaded in any order, and in parallel, but the client must ensure that
         the parts are stored in the correct order by uploading each chunk to the correct upload URL.
 
-        This returns a context object you must enter (using the `with` keyword), then call `upload_part` on
+        This returns a context manager you must enter (using the `with` keyword), then call `upload_part`
         for each part before exiting.
 
         Args:
@@ -833,13 +781,11 @@ class FilesAPI(APIClient):
             overwrite (bool): If 'overwrite' is set to true, and the POST body content specifies a 'externalId' field, fields for the file found for externalId can be overwritten. The default setting is false. If metadata is included in the request body, all of the original metadata will be overwritten. The actual file will be overwritten after successful upload. If there is no successful upload, the current file contents will be kept. File-Asset mappings only change if explicitly stated in the assetIds field of the POST json body. Do not set assetIds in request body if you want to keep the current file-asset mappings.
 
         Returns:
-            FileMultipartUploadSession: Object containing metadata about the created file,
-            and information needed to upload the file content. Use this object to manage the file upload, and `exit` it once
-            all parts are uploaded.
+            FileMultipartUploadSession: Object containing metadata about the created file, and information needed to upload the file content. Use this object to manage the file upload, and `exit` it once all parts are uploaded.
 
         Examples:
 
-            Upload binary data in two chunks
+            Upload binary data in two chunks:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -890,14 +836,14 @@ class FilesAPI(APIClient):
         external_id: str | None = None,
         instance_id: NodeId | None = None,
     ) -> FileMultipartUploadSession:
-        """Begin uploading a file in multiple parts. This allows uploading files larger than 5GiB.
+        """Begin uploading a file in multiple parts whose metadata is already created in CDF. This allows uploading files larger than 5GiB.
         Note that the size of each part may not exceed 4000MiB, and the size of each part except the last
         must be greater than 5MiB.
 
         The file chunks may be uploaded in any order, and in parallel, but the client must ensure that
         the parts are stored in the correct order by uploading each chunk to the correct upload URL.
 
-        This returns a context object you must enter (using the `with` keyword), then call `upload_part` on
+        This returns a context manager you must enter (using the `with` keyword), then call `upload_part`
         for each part before exiting.
 
         Args:
@@ -906,13 +852,11 @@ class FilesAPI(APIClient):
             instance_id (NodeId | None): Instance ID of the file.
 
         Returns:
-            FileMultipartUploadSession: Object containing metadata about the created file,
-            and information needed to upload the file content. Use this object to manage the file upload, and `exit` it once
-            all parts are uploaded.
+            FileMultipartUploadSession: Object containing metadata about the created file, and information needed to upload the file content. Use this object to manage the file upload, and `exit` it once all parts are uploaded.
 
         Examples:
 
-            Upload binary data in two chunks
+            Upload binary data in two chunks:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
@@ -921,18 +865,12 @@ class FilesAPI(APIClient):
                 ...     session.upload_part(0, "hello" * 1_200_000)
                 ...     session.upload_part(1, " world")
         """
-        headers: dict | None = None
-        if instance_id is not None:
-            self._warn_alpha()
-            headers = {"cdf-version": "alpha"}
         identifiers = IdentifierSequence.load(external_ids=external_id, instance_ids=instance_id).as_singleton()
-
         try:
             res = self._post(
                 url_path=f"{self._RESOURCE_PATH}/multiuploadlink",
                 json={"items": identifiers.as_dicts()},
                 params={"parts": parts},
-                headers=headers,
             )
         except CogniteAPIError as e:
             if e.code == 403:
@@ -944,7 +882,7 @@ class FilesAPI(APIClient):
         upload_id = returned_file_metadata["uploadId"]
 
         return FileMultipartUploadSession(
-            FileMetadata.load(returned_file_metadata), upload_urls, upload_id, self._cognite_client
+            FileMetadata._load(returned_file_metadata), upload_urls, upload_id, self._cognite_client
         )
 
     def _upload_multipart_part(self, upload_url: str, content: str | bytes | TextIO | BinaryIO) -> None:
@@ -962,16 +900,12 @@ class FilesAPI(APIClient):
         upload_response = self._http_client_with_retry.request(
             "PUT",
             upload_url,
-            accept="*/*",
             data=content,
             timeout=self._config.file_transfer_timeout,
-            headers=None,
+            headers={"accept": "*/*"},
         )
         if not upload_response.ok:
-            raise CogniteFileUploadError(
-                message=upload_response.text,
-                code=upload_response.status_code,
-            )
+            raise CogniteFileUploadError(message=upload_response.text, code=upload_response.status_code)
 
     def _complete_multipart_upload(self, session: FileMultipartUploadSession) -> None:
         """Complete a multipart upload. Once this returns the file can be downloaded.
@@ -979,13 +913,9 @@ class FilesAPI(APIClient):
         Args:
             session (FileMultipartUploadSession): Multipart upload session returned from
         """
-        headers: dict | None = None
-        if session.file_metadata.instance_id is not None:
-            headers = {"cdf-version": "alpha"}
         self._post(
             self._RESOURCE_PATH + "/completemultipartupload",
             json={"id": session.file_metadata.id, "uploadId": session._upload_id},
-            headers=headers,
         )
 
     def retrieve_download_urls(
@@ -1006,11 +936,6 @@ class FilesAPI(APIClient):
         Returns:
             dict[int | str | NodeId, str]: Dictionary containing download urls.
         """
-
-        headers: dict | None = None
-        if instance_id is not None:
-            self._warn_alpha()
-            headers = {"cdf-version": "alpha"}
         identifiers = IdentifierSequence.load(ids=id, external_ids=external_id, instance_ids=instance_id)
 
         batch_size = 100
@@ -1019,7 +944,7 @@ class FilesAPI(APIClient):
         if extended_expiration:
             query_params["extendedExpiration"] = True
         tasks = [
-            dict(url_path="/files/downloadlink", json={"items": id_batch}, params=query_params, headers=headers)
+            {"url_path": "/files/downloadlink", "json": {"items": id_batch}, "params": query_params}
             for id_batch in id_batches
         ]
         tasks_summary = execute_tasks(self._post, tasks, max_workers=self._config.max_workers)
@@ -1047,7 +972,7 @@ class FilesAPI(APIClient):
 
             file_suffixes = Path(file_name).suffixes
             file_postfix = "".join(file_suffixes)
-            file_base = file_name[: file_name.index(file_postfix) or None]  # Awaiting .removesuffix in 3.9:
+            file_base = file_name.removesuffix(file_postfix)
 
             new_name = file_name
             while (new_name in unique_created) or (new_name in unique_original):
@@ -1091,20 +1016,16 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Download files by id and external id into directory 'my_directory'::
+            Download files by id and external id into directory 'my_directory':
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
                 >>> client.files.download(directory="my_directory", id=[1,2,3], external_id=["abc", "def"])
 
-            Download files by id to the current directory::
+            Download files by id to the current directory:
 
                 >>> client.files.download(directory=".", id=[1,2,3])
         """
-        headers: dict | None = None
-        if instance_id is not None:
-            self._warn_alpha()
-            headers = {"cdf-version": "alpha"}
         identifiers = IdentifierSequence.load(ids=id, external_ids=external_id, instance_ids=instance_id)
 
         directory = Path(directory)
@@ -1117,7 +1038,6 @@ class FilesAPI(APIClient):
         all_ids, filepaths, directories = self._get_ids_filepaths_directories(
             directory, id_to_metadata, keep_directory_structure
         )
-
         if keep_directory_structure:
             for file_folder in set(directories):
                 file_folder.mkdir(parents=True, exist_ok=True)
@@ -1127,7 +1047,7 @@ class FilesAPI(APIClient):
             filepaths = [Path(file_path) for file_path in filepaths_str]
 
         self._download_files_to_directory(
-            directory=directory, all_ids=all_ids, id_to_metadata=id_to_metadata, filepaths=filepaths, headers=headers
+            directory=directory, all_ids=all_ids, id_to_metadata=id_to_metadata, filepaths=filepaths
         )
 
     @staticmethod
@@ -1168,14 +1088,7 @@ class FilesAPI(APIClient):
         ids = [id["id"] for id in all_ids if "id" in id]
         external_ids = [id["externalId"] for id in all_ids if "externalId" in id]
         instance_ids = [id["instanceId"] for id in all_ids if "instanceId" in id]
-
-        files_metadata = self.retrieve_multiple(ids=ids, external_ids=external_ids, instance_ids=instance_ids)
-
-        id_to_metadata: dict[int, FileMetadata] = {}
-        for f in files_metadata:
-            id_to_metadata[f.id] = f
-
-        return id_to_metadata
+        return self.retrieve_multiple(ids=ids, external_ids=external_ids, instance_ids=instance_ids)._id_to_item
 
     def _download_files_to_directory(
         self,
@@ -1193,10 +1106,9 @@ class FilesAPI(APIClient):
             str_format_element_fn=lambda metadata: metadata.id,
         )
 
-    def _get_download_link(self, identifier: dict[str, int | str], headers: dict | None = None) -> str:
-        return self._post(url_path="/files/downloadlink", json={"items": [identifier]}, headers=headers).json()[
-            "items"
-        ][0]["downloadUrl"]
+    def _get_download_link(self, identifier: dict[str, int | str]) -> str:
+        (item,) = self._post(url_path="/files/downloadlink", json={"items": [identifier]}).json()["items"]
+        return item["downloadUrl"]
 
     def _process_file_download(
         self,
@@ -1209,12 +1121,12 @@ class FilesAPI(APIClient):
         file_is_in_download_directory = directory.resolve() in file_path_absolute.parents
         if not file_is_in_download_directory:
             raise RuntimeError(f"Resolved file path '{file_path_absolute}' is not inside download directory")
-        download_link = self._get_download_link(identifier, headers)
+        download_link = self._get_download_link(identifier)
         self._download_file_to_path(download_link, file_path_absolute)
 
     def _download_file_to_path(self, download_link: str, path: Path, chunk_size: int = 2**21) -> None:
         with self._http_client_with_retry.request(
-            "GET", download_link, accept="*/*", stream=True, timeout=self._config.file_transfer_timeout
+            "GET", download_link, headers={"accept": "*/*"}, stream=True, timeout=self._config.file_transfer_timeout
         ) as r:
             with path.open("wb") as f:
                 for chunk in r.iter_content(chunk_size=chunk_size):
@@ -1239,17 +1151,12 @@ class FilesAPI(APIClient):
                 >>> client = CogniteClient()
                 >>> client.files.download_to_path("~/mydir/my_downloaded_file.txt", id=123)
         """
-        headers: dict | None = None
-        if instance_id is not None:
-            self._warn_alpha()
-            headers = {"cdf-version": "alpha"}
-
         if isinstance(path, str):
             path = Path(path)
         if not path.parent.is_dir():
-            raise NotADirectoryError(f"{path.parent} is not a directory")
+            raise NotADirectoryError(str(path.parent))
         identifier = Identifier.of_either(id, external_id, instance_id).as_dict()
-        download_link = self._get_download_link(identifier, headers)
+        download_link = self._get_download_link(identifier)
         self._download_file_to_path(download_link, path)
 
     def download_bytes(
@@ -1264,26 +1171,22 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            Download a file's content into memory::
+            Download a file's content into memory:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
                 >>> file_content = client.files.download_bytes(id=1)
 
         Returns:
-            bytes: No description."""
-        headers: dict | None = None
-        if instance_id is not None:
-            self._warn_alpha()
-            headers = {"cdf-version": "alpha"}
-
+            bytes: The file in binary format
+        """
         identifier = Identifier.of_either(id, external_id, instance_id).as_dict()
-        download_link = self._get_download_link(identifier, headers)
+        download_link = self._get_download_link(identifier)
         return self._download_file(download_link)
 
     def _download_file(self, download_link: str) -> bytes:
         res = self._http_client_with_retry.request(
-            "GET", download_link, accept="*/*", timeout=self._config.file_transfer_timeout
+            "GET", download_link, headers={"accept": "*/*"}, timeout=self._config.file_transfer_timeout
         )
         return res.content
 
@@ -1343,39 +1246,31 @@ class FilesAPI(APIClient):
 
         Examples:
 
-            List files metadata and filter on external id prefix::
+            List files metadata and filter on external id prefix:
 
                 >>> from cognite.client import CogniteClient
                 >>> client = CogniteClient()
                 >>> file_list = client.files.list(limit=5, external_id_prefix="prefix")
 
-            Iterate over files metadata::
+            Iterate over files metadata:
 
-                >>> from cognite.client import CogniteClient
-                >>> client = CogniteClient()
                 >>> for file_metadata in client.files:
                 ...     file_metadata # do something with the file metadata
 
-            Iterate over chunks of files metadata to reduce memory load::
+            Iterate over chunks of files metadata to reduce memory load:
 
-                >>> from cognite.client import CogniteClient
-                >>> client = CogniteClient()
                 >>> for file_list in client.files(chunk_size=2500):
                 ...     file_list # do something with the files
 
-            Filter files based on labels::
+            Filter files based on labels:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import LabelFilter
-                >>> client = CogniteClient()
                 >>> my_label_filter = LabelFilter(contains_all=["WELL LOG", "VERIFIED"])
                 >>> file_list = client.files.list(labels=my_label_filter)
 
-            Filter files based on geoLocation::
+            Filter files based on geoLocation:
 
-                >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import GeoLocationFilter, GeometryFilter
-                >>> client = CogniteClient()
                 >>> my_geo_location_filter = GeoLocationFilter(relation="intersects", shape=GeometryFilter(type="Point", coordinates=[35,10]))
                 >>> file_list = client.files.list(geo_location=my_geo_location_filter)
         """
