@@ -7,7 +7,7 @@ import logging
 import re
 import warnings
 from collections import UserList
-from collections.abc import Iterator, MutableMapping, Sequence
+from collections.abc import Iterator, Mapping, MutableMapping, Sequence
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -41,7 +41,7 @@ from cognite.client.data_classes._base import (
 )
 from cognite.client.data_classes.aggregations import AggregationFilter, UniqueResultList
 from cognite.client.data_classes.filters import Filter
-from cognite.client.exceptions import CogniteAPIError, CogniteNotFoundError
+from cognite.client.exceptions import CogniteAPIError, CogniteNotFoundError, CogniteProjectAccessError
 from cognite.client.utils import _json
 from cognite.client.utils._auxiliary import (
     get_current_sdk_version,
@@ -72,6 +72,8 @@ logger = logging.getLogger(__name__)
 
 T = TypeVar("T", bound=CogniteObject)
 
+VALID_AGGREGATIONS = {"count", "cardinalityValues", "cardinalityProperties", "uniqueValues", "uniqueProperties"}
+
 
 class APIClient:
     _RESOURCE_PATH: str
@@ -99,10 +101,14 @@ class APIClient:
                     "geospatial/(compute|crs/byids|featuretypes/(byids|list))",
                     "geospatial/featuretypes/[A-Za-z][A-Za-z0-9_]{0,31}/features/(aggregate|list|byids|search|search-streaming|[A-Za-z][A-Za-z0-9_]{0,255}/rasters/[A-Za-z][A-Za-z0-9_]{0,31})",
                     "transformations/(filter|byids|jobs/byids|schedules/byids|query/run)",
+                    "simulators/list",
                     "extpipes/(list|byids|runs/list)",
                     "workflows/.*",
                     "hostedextractors/.*",
                     "postgresgateway/.*",
+                    "context/diagram/.*",
+                    "ai/tools/documents/(summarize|ask)",
+                    "ai/agents(/(byids|delete))?",
                 )
             )
         ]
@@ -226,18 +232,24 @@ class APIClient:
         kwargs.setdefault("allow_redirects", False)
 
         if is_retryable:
-            res = self._http_client_with_retry.request(method=method, url=full_url, accept=accept, **kwargs)
+            res = self._http_client_with_retry.request(method=method, url=full_url, **kwargs)
         else:
-            res = self._http_client.request(method=method, url=full_url, accept=accept, **kwargs)
+            res = self._http_client.request(method=method, url=full_url, **kwargs)
+
+        match res.status_code:
+            case 200 | 201 | 202 | 204:
+                pass
+            case 401:
+                self._raise_no_project_access_error(res)
+            case _:
+                self._raise_api_error(res, payload=json_payload)
 
-        if not self._status_ok(res.status_code):
-            self._raise_api_error(res, payload=json_payload)
         stream = kwargs.get("stream")
         self._log_request(res, payload=json_payload, stream=stream)
         return res
 
     def _configure_headers(
-        self, accept: str, additional_headers: dict[str, str], api_subversion: str | None
+        self, accept: str, additional_headers: dict[str, str], api_subversion: str | None = None
     ) -> MutableMapping[str, Any]:
         headers: MutableMapping[str, Any] = CaseInsensitiveDict()
         headers.update(requests.utils.default_headers())
@@ -278,7 +290,7 @@ class APIClient:
         if method not in valid_methods:
             raise ValueError(f"Method {method} is not valid. Must be one of {valid_methods}")
 
-        return method in ["GET", "PUT", "PATCH"] or method == "POST" and self._url_is_retryable(path)
+        return method in ["GET", "PUT", "PATCH"] or (method == "POST" and self._url_is_retryable(path))
 
     @classmethod
     @functools.lru_cache(64)
@@ -780,11 +792,8 @@ class APIClient:
         api_subversion: str | None = None,
     ) -> int | UniqueResultList:
         verify_limit(limit)
-        if aggregate not in ["count", "cardinalityValues", "cardinalityProperties", "uniqueValues", "uniqueProperties"]:
-            raise ValueError(
-                f"Invalid aggregate '{aggregate}'. Valid aggregates are 'count', 'cardinalityValues', "
-                f"'cardinalityProperties', 'uniqueValues', and 'uniqueProperties'."
-            )
+        if aggregate not in VALID_AGGREGATIONS:
+            raise ValueError(f"Invalid aggregate {aggregate!r}. Valid aggregates are {sorted(VALID_AGGREGATIONS)}.")
 
         body: dict[str, Any] = {"aggregate": aggregate}
         if properties is not None:
@@ -792,12 +801,13 @@ class APIClient:
                 properties, property_aggregation_filter = properties
             else:
                 property_aggregation_filter = None
+
             if isinstance(properties, EnumProperty):
                 dumped_properties = properties.as_reference()
             elif isinstance(properties, str):
                 dumped_properties = [to_camel_case(properties)]
             elif isinstance(properties, list):
-                dumped_properties = [to_camel_case(p) for p in properties]
+                dumped_properties = [to_camel_case(properties[0])] if len(properties) == 1 else properties
             else:
                 raise ValueError(f"Unknown property format: {properties}")
 
@@ -825,8 +835,6 @@ class APIClient:
                 dumped_filter = filter.dump(camel_case=True)
             elif isinstance(filter, dict):
                 dumped_filter = convert_all_keys_to_camel_case(filter)
-            else:
-                raise ValueError(f"Unknown filter format: {filter}")
             body["filter"] = dumped_filter
 
         if advanced_filter is not None:
@@ -960,11 +968,12 @@ class APIClient:
         extra_body_fields: dict[str, Any] | None = None,
         returns_items: bool = False,
         executor: TaskExecutor | None = None,
+        delete_endpoint: str = "/delete",
     ) -> list | None:
-        resource_path = resource_path or self._RESOURCE_PATH
+        resource_path = (resource_path or self._RESOURCE_PATH) + delete_endpoint
         tasks = [
             {
-                "url_path": resource_path + "/delete",
+                "url_path": resource_path,
                 "json": {
                     "items": chunk.as_dicts() if wrap_ids else chunk.as_primitives(),
                     **(extra_body_fields or {}),
@@ -987,7 +996,7 @@ class APIClient:
     @overload
     def _update_multiple(
         self,
-        items: CogniteResource | CogniteUpdate,
+        items: CogniteResource | CogniteUpdate | WriteableCogniteResource,
         list_cls: type[T_CogniteResourceList],
         resource_cls: type[T_CogniteResource],
         update_cls: type[CogniteUpdate],
@@ -996,12 +1005,13 @@ class APIClient:
         headers: dict[str, Any] | None = None,
         mode: Literal["replace_ignore_null", "patch", "replace"] = "replace_ignore_null",
         api_subversion: str | None = None,
+        cdf_item_by_id: Mapping[Any, T_CogniteResource] | None = None,
     ) -> T_CogniteResource: ...
 
     @overload
     def _update_multiple(
         self,
-        items: Sequence[CogniteResource | CogniteUpdate],
+        items: Sequence[CogniteResource | CogniteUpdate | WriteableCogniteResource],
         list_cls: type[T_CogniteResourceList],
         resource_cls: type[T_CogniteResource],
         update_cls: type[CogniteUpdate],
@@ -1010,11 +1020,15 @@ class APIClient:
         headers: dict[str, Any] | None = None,
         mode: Literal["replace_ignore_null", "patch", "replace"] = "replace_ignore_null",
         api_subversion: str | None = None,
+        cdf_item_by_id: Mapping[Any, T_CogniteResource] | None = None,
     ) -> T_CogniteResourceList: ...
 
     def _update_multiple(
         self,
-        items: Sequence[CogniteResource | CogniteUpdate] | CogniteResource | CogniteUpdate,
+        items: Sequence[CogniteResource | CogniteUpdate | WriteableCogniteResource]
+        | CogniteResource
+        | CogniteUpdate
+        | WriteableCogniteResource,
         list_cls: type[T_CogniteResourceList],
         resource_cls: type[T_CogniteResource],
         update_cls: type[CogniteUpdate],
@@ -1023,6 +1037,7 @@ class APIClient:
         headers: dict[str, Any] | None = None,
         mode: Literal["replace_ignore_null", "patch", "replace"] = "replace_ignore_null",
         api_subversion: str | None = None,
+        cdf_item_by_id: Mapping[Any, T_CogniteResource] | None = None,
     ) -> T_CogniteResourceList | T_CogniteResource:
         resource_path = resource_path or self._RESOURCE_PATH
         patch_objects = []
@@ -1036,9 +1051,7 @@ class APIClient:
             if isinstance(item, CogniteResource):
                 patch_objects.append(
                     self._convert_resource_to_patch_object(
-                        item,
-                        update_cls._get_update_properties(item),
-                        mode,
+                        item, update_cls._get_update_properties(item), mode, cdf_item_by_id
                     )
                 )
             elif isinstance(item, CogniteUpdate):
@@ -1077,6 +1090,7 @@ class APIClient:
         mode: Literal["patch", "replace"],
         input_resource_cls: type[CogniteResource] | None = None,
         api_subversion: str | None = None,
+        cdf_item_by_id: Mapping[Any, T_CogniteResource] | None = None,
     ) -> T_WritableCogniteResource | T_CogniteResourceList:
         if mode not in ["patch", "replace"]:
             raise ValueError(f"mode must be either 'patch' or 'replace', got {mode!r}")
@@ -1084,7 +1098,13 @@ class APIClient:
         items = cast(Sequence[T_WritableCogniteResource], [items] if is_single else items)
         try:
             result = self._update_multiple(
-                items, list_cls, resource_cls, update_cls, mode=mode, api_subversion=api_subversion
+                items,
+                list_cls,
+                resource_cls,
+                update_cls,
+                mode=mode,
+                api_subversion=api_subversion,
+                cdf_item_by_id=cast(Mapping | None, cdf_item_by_id),
             )
         except CogniteNotFoundError as not_found_error:
             items_by_external_id = {item.external_id: item for item in items if item.external_id is not None}  # type: ignore [attr-defined]
@@ -1127,11 +1147,12 @@ class APIClient:
                         update_cls=update_cls,
                         mode=mode,
                         api_subversion=api_subversion,
+                        cdf_item_by_id=cast(Mapping | None, cdf_item_by_id),
                     )
             except CogniteAPIError as api_error:
-                successful = list(api_error.successful)
-                unknown = list(api_error.unknown)
-                failed = list(api_error.failed)
+                successful = api_error.successful
+                unknown = api_error.unknown
+                failed = api_error.failed
 
                 successful.extend(not_found_error.successful)
                 unknown.extend(not_found_error.unknown)
@@ -1162,7 +1183,6 @@ class APIClient:
             result = list_cls(
                 (successful_resources or []) + (created or []) + (updated or []), cognite_client=self._cognite_client
             )
-
             # Reorder to match the order of the input items
             result.data = [
                 result.get(
@@ -1224,6 +1244,7 @@ class APIClient:
         resource: CogniteResource,
         update_attributes: list[PropertySpec],
         mode: Literal["replace_ignore_null", "patch", "replace"] = "replace_ignore_null",
+        cdf_item_by_id: Mapping[Any, T_CogniteResource] | None = None,
     ) -> dict[str, dict[str, dict]]:
         dumped = resource.dump(camel_case=True)
 
@@ -1257,8 +1278,10 @@ class APIClient:
         for prop in update_attributes:
             if prop.is_beta:
                 continue
+            elif prop.is_explicit_nullable_object:
+                clear_with: dict = {"setNull": True}
             elif prop.is_object:
-                clear_with: dict = {"set": {}}
+                clear_with = {"set": {}}
             elif prop.is_list:
                 clear_with = {"set": []}
             elif prop.is_nullable:
@@ -1268,9 +1291,13 @@ class APIClient:
             cleared[to_camel_case(prop.name)] = clear_with
         return cleared
 
-    @staticmethod
-    def _status_ok(status_code: int) -> bool:
-        return status_code in {200, 201, 202, 204}
+    def _raise_no_project_access_error(self, res: Response) -> NoReturn:
+        raise CogniteProjectAccessError(
+            client=self._cognite_client,
+            project=self._cognite_client._config.project,
+            x_request_id=res.headers.get("X-Request-Id"),
+            cluster=self._config.cdf_cluster,
+        )
 
     def _raise_api_error(self, res: Response, payload: dict) -> NoReturn:
         x_request_id = res.headers.get("X-Request-Id")
@@ -1312,6 +1339,7 @@ class APIClient:
                     f"REDIRECT AFTER HTTP Error {res_hist.status_code} {res_hist.request.method} {res_hist.request.url}: {res_hist.content.decode()}"
                 )
         logger.debug(f"HTTP Error {code} {res.request.method} {res.request.url}: {msg}", extra=error_details)
+        # TODO: We should throw "CogniteNotFoundError" if missing is populated and CogniteDuplicatedError when duplicated...
         raise CogniteAPIError(
             msg,
             code,
