@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import warnings
 from abc import ABC, abstractmethod
 from collections import UserList
 from collections.abc import Collection, Iterable, Iterator, Sequence
@@ -26,7 +27,7 @@ from typing_extensions import Self
 from cognite.client.exceptions import CogniteMissingClientError
 from cognite.client.utils import _json
 from cognite.client.utils._auxiliary import fast_dict_load, load_resource_to_dict, load_yaml_or_json
-from cognite.client.utils._identifier import IdentifierSequence
+from cognite.client.utils._identifier import IdentifierSequence, InstanceId
 from cognite.client.utils._importing import local_import
 from cognite.client.utils._pandas_helpers import (
     convert_nullable_int_cols,
@@ -141,7 +142,7 @@ class CogniteObject:
             str: A YAML formatted string representing the instance.
         """
         yaml = local_import("yaml")
-        return yaml.dump(self.dump(camel_case=True), sort_keys=False)
+        return yaml.safe_dump(self.dump(camel_case=True), sort_keys=False)
 
     @final
     @classmethod
@@ -264,13 +265,15 @@ class CogniteResourceList(UserList, Generic[T_CogniteResource], _WithClientMixin
         self._build_id_mappings()
 
     def _build_id_mappings(self) -> None:
-        self._id_to_item, self._external_id_to_item = {}, {}
+        self._id_to_item, self._external_id_to_item, self._instance_id_to_item = {}, {}, {}
         if not self.data:
             return
         if hasattr(self.data[0], "external_id"):
             self._external_id_to_item = {item.external_id: item for item in self.data if item.external_id is not None}
         if hasattr(self.data[0], "id"):
             self._id_to_item = {item.id: item for item in self.data if item.id is not None}
+        if hasattr(self.data[0], "instance_id"):
+            self._instance_id_to_item = {item.instance_id: item for item in self.data if item.instance_id is not None}
 
     def pop(self, i: int = -1) -> T_CogniteResource:
         return super().pop(i)
@@ -303,6 +306,7 @@ class CogniteResourceList(UserList, Generic[T_CogniteResource], _WithClientMixin
             super().extend(other)
             self._external_id_to_item.update(other_res_list._external_id_to_item)
             self._id_to_item.update(other_res_list._id_to_item)
+            self._instance_id_to_item.update(other_res_list._instance_id_to_item)
         else:
             raise ValueError("Unable to extend as this would introduce duplicates")
 
@@ -324,22 +328,30 @@ class CogniteResourceList(UserList, Generic[T_CogniteResource], _WithClientMixin
             str: A YAML formatted string representing the instances.
         """
         yaml = local_import("yaml")
-        return yaml.dump(self.dump(camel_case=True), sort_keys=False)
+        return yaml.safe_dump(self.dump(camel_case=True), sort_keys=False)
 
-    def get(self, id: int | None = None, external_id: str | None = None) -> T_CogniteResource | None:
-        """Get an item from this list by id or external_id.
+    def get(
+        self,
+        id: int | None = None,
+        external_id: str | None = None,
+        instance_id: InstanceId | tuple[str, str] | None = None,
+    ) -> T_CogniteResource | None:
+        """Get an item from this list by id, external_id or instance_id.
 
         Args:
             id (int | None): The id of the item to get.
             external_id (str | None): The external_id of the item to get.
+            instance_id (InstanceId | tuple[str, str] | None): The instance_id of the item to get.
 
         Returns:
-            T_CogniteResource | None: The requested item
+            T_CogniteResource | None: The requested item if present, otherwise None.
         """
-        IdentifierSequence.load(id, external_id).assert_singleton()
+        (ident := IdentifierSequence.load(id, external_id, instance_id)).assert_singleton()
         if id:
             return self._id_to_item.get(id)
-        return self._external_id_to_item.get(external_id)
+        elif external_id:
+            return self._external_id_to_item.get(external_id)
+        return self._instance_id_to_item.get(ident.as_primitives()[0])
 
     def to_pandas(
         self,
@@ -435,6 +447,8 @@ class PropertySpec:
     is_nullable: bool = True
     # Used to skip replace when the value is None
     is_beta: bool = False
+    # Objects that are nullable and support setNull. This is hosted extractor mqtt/kafka sources
+    is_explicit_nullable_object: bool = False
 
     def __post_init__(self) -> None:
         assert not (self.is_list and self.is_object), "PropertySpec cannot be both list and object"
@@ -446,6 +460,13 @@ class CogniteUpdate:
         self._external_id = external_id
         self._update_object: dict[str, Any] = {}
 
+        if id is not None and external_id is not None:
+            warnings.warn(
+                "Update object got both of 'id' and 'external_id', 'external_id' will be ignored.",
+                UserWarning,
+                stacklevel=2,
+            )
+
     def __eq__(self, other: Any) -> bool:
         return type(self) is type(other) and self.dump() == other.dump()
 
