@@ -6,48 +6,50 @@ import operator as op
 import textwrap
 import threading
 import warnings
+from abc import ABC
 from collections import Counter, defaultdict
+from collections.abc import Sequence
+from enum import auto
 from functools import lru_cache
+from graphlib import TopologicalSorter
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
     Any,
-    Collection,
-    Dict,
-    List,
     Literal,
-    Optional,
-    Sequence,
     TextIO,
-    Union,
+    TypeAlias,
+    TypeVar,
     cast,
 )
 
-from graphlib import TopologicalSorter
-from typing_extensions import TypeAlias
-
 from cognite.client.data_classes._base import (
     CogniteFilter,
     CogniteLabelUpdate,
     CogniteListUpdate,
+    CogniteObject,
     CogniteObjectUpdate,
     CognitePrimitiveUpdate,
-    CognitePropertyClassUtil,
     CogniteResource,
     CogniteResourceList,
     CogniteSort,
     CogniteUpdate,
     EnumProperty,
+    ExternalIDTransformerMixin,
     IdTransformerMixin,
     PropertySpec,
+    WriteableCogniteResource,
+    WriteableCogniteResourceList,
 )
-from cognite.client.data_classes.labels import Label, LabelDefinition, LabelFilter
+from cognite.client.data_classes.labels import Label, LabelDefinition, LabelDefinitionWrite, LabelFilter
 from cognite.client.data_classes.shared import GeoLocation, GeoLocationFilter, TimestampRange
 from cognite.client.exceptions import CogniteAssetHierarchyError
-from cognite.client.utils._auxiliary import split_into_chunks
+from cognite.client.utils._auxiliary import remove_duplicates_keep_order, split_into_chunks
 from cognite.client.utils._concurrency import execute_tasks
 from cognite.client.utils._graph import find_all_cycles_with_elements
-from cognite.client.utils._text import DrawTables, shorten
+from cognite.client.utils._importing import local_import
+from cognite.client.utils._text import DrawTables, convert_dict_to_case, shorten
+from cognite.client.utils.useful_types import SequenceNotStr
 
 if TYPE_CHECKING:
     import pandas
@@ -57,29 +59,14 @@ if TYPE_CHECKING:
     from cognite.client.data_classes._base import T_CogniteResource, T_CogniteResourceList
 
 
-class AssetAggregate(dict):
-    """Aggregation group of assets
-
-    Args:
-        count (int | None): Size of the aggregation group
-        **kwargs (Any): No description.
-    """
-
-    def __init__(self, count: int | None = None, **kwargs: Any) -> None:
-        self.count = count
-        self.update(kwargs)
-
-    count = CognitePropertyClassUtil.declare_property("count")
-
-
-class AggregateResultItem(dict):
+class AggregateResultItem(CogniteObject):
     """Aggregated metrics of the asset
 
     Args:
         child_count (int | None): Number of direct descendants for the asset
         depth (int | None): Asset path depth (number of levels below root node).
         path (list[dict[str, Any]] | None): IDs of assets on the path to the asset.
-        **kwargs (Any): No description.
+        **_ (Any): No description.
     """
 
     def __init__(
@@ -87,20 +74,16 @@ class AggregateResultItem(dict):
         child_count: int | None = None,
         depth: int | None = None,
         path: list[dict[str, Any]] | None = None,
-        **kwargs: Any,
+        **_: Any,
     ) -> None:
         self.child_count = child_count
         self.depth = depth
         self.path = path
-        self.update(kwargs)
-
-    child_count = CognitePropertyClassUtil.declare_property("childCount")
-    depth = CognitePropertyClassUtil.declare_property("depth")
-    path = CognitePropertyClassUtil.declare_property("path")
 
 
-class Asset(CogniteResource):
-    """A representation of a physical asset, for example a factory or a piece of equipment.
+class AssetCore(WriteableCogniteResource["AssetWrite"], ABC):
+    """A representation of a physical asset, for example, a factory or a piece of equipment. This
+    is the parent class for the Asset and AssetWrite classes.
 
     Args:
         external_id (str | None): The external ID provided by the client. Must be unique for the resource type.
@@ -109,7 +92,70 @@ class Asset(CogniteResource):
         parent_external_id (str | None): The external ID of the parent. The property is omitted if the asset doesn't have a parent or if the parent doesn't have externalId.
         description (str | None): The description of the asset.
         data_set_id (int | None): The id of the dataset this asset belongs to.
-        metadata (dict[str, str] | None): Custom, application specific metadata. String key -> String value. Limits: Maximum length of key is 128 bytes, value 10240 bytes, up to 256 key-value pairs, of total size at most 10240.
+        metadata (dict[str, str] | None): Custom, application-specific metadata. String key -> String value. Limits: Maximum length of key is 128 bytes, value 10240 bytes, up to 256 key-value pairs, of total size at most 10240.
+        source (str | None): The source of the asset.
+        labels (list[Label] | None): A list of the labels associated with this resource item.
+        geo_location (GeoLocation | None): The geographic metadata of the asset.
+    """
+
+    def __init__(
+        self,
+        external_id: str | None = None,
+        name: str | None = None,
+        parent_id: int | None = None,
+        parent_external_id: str | None = None,
+        description: str | None = None,
+        data_set_id: int | None = None,
+        metadata: dict[str, str] | None = None,
+        source: str | None = None,
+        labels: list[Label] | None = None,
+        geo_location: GeoLocation | None = None,
+    ) -> None:
+        if geo_location is not None and not isinstance(geo_location, GeoLocation):
+            raise TypeError("Asset.geo_location should be of type GeoLocation")
+        self.external_id = external_id
+        self.name = name
+        self.parent_id = parent_id
+        self.parent_external_id = parent_external_id
+        self.description = description
+        self.data_set_id = data_set_id
+        self.metadata = metadata
+        self.source = source
+        self.labels = labels
+        self.geo_location = geo_location
+
+    @classmethod
+    def _load(cls: type[T_Asset], resource: dict, cognite_client: CogniteClient | None = None) -> T_Asset:
+        instance = super()._load(resource, cognite_client)
+        instance.labels = Label._load_list(instance.labels)
+        if isinstance(instance.geo_location, dict):
+            instance.geo_location = GeoLocation._load(instance.geo_location)
+        return instance
+
+    def dump(self, camel_case: bool = True) -> dict[str, Any]:
+        result = super().dump(camel_case)
+        if self.labels is not None:
+            result["labels"] = [label.dump(camel_case) for label in self.labels]
+        if self.geo_location is not None:
+            result["geoLocation" if camel_case else "geo_location"] = self.geo_location.dump(camel_case)
+        return result
+
+
+T_Asset = TypeVar("T_Asset", bound=AssetCore)
+
+
+class Asset(AssetCore):
+    """A representation of a physical asset, for example, a factory or a piece of equipment. This
+    is the read version of the Asset class, it is used when retrieving assets from the Cognite API.
+
+    Args:
+        external_id (str | None): The external ID provided by the client. Must be unique for the resource type.
+        name (str | None): The name of the asset.
+        parent_id (int | None): The parent of the node, null if it is the root node.
+        parent_external_id (str | None): The external ID of the parent. The property is omitted if the asset doesn't have a parent or if the parent doesn't have externalId.
+        description (str | None): The description of the asset.
+        data_set_id (int | None): The id of the dataset this asset belongs to.
+        metadata (dict[str, str] | None): Custom, application-specific metadata. String key -> String value. Limits: Maximum length of key is 128 bytes, value 10240 bytes, up to 256 key-value pairs, of total size at most 10240.
         source (str | None): The source of the asset.
         labels (list[Label | str | LabelDefinition | dict] | None): A list of the labels associated with this resource item.
         geo_location (GeoLocation | None): The geographic metadata of the asset.
@@ -117,7 +163,7 @@ class Asset(CogniteResource):
         created_time (int | None): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
         last_updated_time (int | None): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
         root_id (int | None): ID of the root asset.
-        aggregates (dict[str, Any] | AggregateResultItem | None): Aggregated metrics of the asset
+        aggregates (AggregateResultItem | dict[str, Any] | None): Aggregated metrics of the asset
         cognite_client (CogniteClient | None): The client to associate with this object.
     """
 
@@ -137,38 +183,57 @@ class Asset(CogniteResource):
         created_time: int | None = None,
         last_updated_time: int | None = None,
         root_id: int | None = None,
-        aggregates: dict[str, Any] | AggregateResultItem | None = None,
+        aggregates: AggregateResultItem | dict[str, Any] | None = None,
         cognite_client: CogniteClient | None = None,
     ) -> None:
-        if geo_location is not None and not isinstance(geo_location, GeoLocation):
-            raise TypeError("Asset.geo_location should be of type GeoLocation")
-        self.external_id = external_id
-        self.name = name
-        self.parent_id = parent_id
-        self.parent_external_id = parent_external_id
-        self.description = description
-        self.data_set_id = data_set_id
-        self.metadata = metadata
-        self.source = source
-        self.labels = Label._load_list(labels)
-        self.geo_location = geo_location
-        self.id = id
-        self.created_time = created_time
-        self.last_updated_time = last_updated_time
+        super().__init__(
+            external_id=external_id,
+            name=name,
+            parent_id=parent_id,
+            parent_external_id=parent_external_id,
+            description=description,
+            data_set_id=data_set_id,
+            metadata=metadata,
+            source=source,
+            labels=Label._load_list(labels),
+            geo_location=geo_location,
+        )
+        # id/created_time/last_updated_time are required when using the class to read,
+        # but don't make sense passing in when creating a new object. So in order to make the typing
+        # correct here (i.e. int and not Optional[int]), we force the type to be int rather than
+        # Optional[int].
+        # TODO: In the next major version we can make these properties required in the constructor
+        self.id: int = id  # type: ignore
+        self.created_time: int = created_time  # type: ignore
+        self.last_updated_time: int = last_updated_time  # type: ignore
         self.root_id = root_id
         self.aggregates = aggregates
         self._cognite_client = cast("CogniteClient", cognite_client)
 
     @classmethod
-    def _load(cls, resource: dict | str, cognite_client: CogniteClient | None = None) -> Asset:
+    def _load(cls, resource: dict, cognite_client: CogniteClient | None = None) -> Asset:
         instance = super()._load(resource, cognite_client)
-        if isinstance(resource, dict) and instance.aggregates is not None:
-            instance.aggregates = AggregateResultItem(**instance.aggregates)
-        instance.labels = Label._load_list(instance.labels)
-        if instance.geo_location is not None:
-            instance.geo_location = GeoLocation._load(instance.geo_location)
+        if isinstance(instance.aggregates, dict):
+            instance.aggregates = AggregateResultItem._load(instance.aggregates)
         return instance
 
+    def as_write(self) -> AssetWrite:
+        """Returns this Asset in its writing version."""
+        if self.name is None:
+            raise ValueError("name is required for the writing version of an asset.")
+        return AssetWrite(
+            external_id=self.external_id,
+            name=self.name,
+            parent_id=self.parent_id,
+            parent_external_id=self.parent_external_id,
+            description=self.description,
+            data_set_id=self.data_set_id,
+            metadata=self.metadata,
+            source=self.source,
+            labels=self.labels,  # type: ignore[arg-type]
+            geo_location=self.geo_location,
+        )
+
     def __hash__(self) -> int:
         return hash(self.external_id)
 
@@ -188,7 +253,8 @@ class Asset(CogniteResource):
         Returns:
             AssetList: The requested assets
         """
-        assert self.id is not None
+        if self.id is None:
+            raise ValueError("Unable to fetch child assets: id is missing")
         return self._cognite_client.assets.list(parent_ids=[self.id], limit=None)
 
     def subtree(self, depth: int | None = None) -> AssetList:
@@ -200,76 +266,166 @@ class Asset(CogniteResource):
         Returns:
             AssetList: The requested assets sorted topologically.
         """
-        assert self.id is not None
+        if self.id is None:
+            raise ValueError("Unable to fetch asset subtree: id is missing")
         return self._cognite_client.assets.retrieve_subtree(id=self.id, depth=depth)
 
     def time_series(self, **kwargs: Any) -> TimeSeriesList:
         """Retrieve all time series related to this asset.
 
         Args:
-            **kwargs (Any): All extra keyword arguments are passed to time_series/list. NB: 'asset_ids' can't be used.
+            **kwargs (Any): All extra keyword arguments are passed to time_series/list.
         Returns:
             TimeSeriesList: All time series related to this asset.
         """
-        assert self.id is not None
-        return self._cognite_client.time_series.list(asset_ids=[self.id], **kwargs)
+        asset_ids = self._prepare_asset_ids("time series", kwargs)
+        return self._cognite_client.time_series.list(asset_ids=asset_ids, **kwargs)
 
     def sequences(self, **kwargs: Any) -> SequenceList:
         """Retrieve all sequences related to this asset.
 
         Args:
-            **kwargs (Any): All extra keyword arguments are passed to sequences/list. NB: 'asset_ids' can't be used.
+            **kwargs (Any): All extra keyword arguments are passed to sequences/list.
         Returns:
             SequenceList: All sequences related to this asset.
         """
-        assert self.id is not None
-        return self._cognite_client.sequences.list(asset_ids=[self.id], **kwargs)
+        asset_ids = self._prepare_asset_ids("sequences", kwargs)
+        return self._cognite_client.sequences.list(asset_ids=asset_ids, **kwargs)
 
     def events(self, **kwargs: Any) -> EventList:
         """Retrieve all events related to this asset.
 
         Args:
-            **kwargs (Any): All extra keyword arguments are passed to events/list. NB: 'asset_ids' can't be used.
+            **kwargs (Any): All extra keyword arguments are passed to events/list.
         Returns:
             EventList: All events related to this asset.
         """
-        assert self.id is not None
-        return self._cognite_client.events.list(asset_ids=[self.id], **kwargs)
+        asset_ids = self._prepare_asset_ids("events", kwargs)
+        return self._cognite_client.events.list(asset_ids=asset_ids, **kwargs)
 
     def files(self, **kwargs: Any) -> FileMetadataList:
         """Retrieve all files metadata related to this asset.
 
         Args:
-            **kwargs (Any): All extra keyword arguments are passed to files/list. NB: 'asset_ids' can't be used.
+            **kwargs (Any): All extra keyword arguments are passed to files/list.
         Returns:
             FileMetadataList: Metadata about all files related to this asset.
         """
-        assert self.id is not None
-        return self._cognite_client.files.list(asset_ids=[self.id], **kwargs)
+        asset_ids = self._prepare_asset_ids("files", kwargs)
+        return self._cognite_client.files.list(asset_ids=asset_ids, **kwargs)
 
-    def dump(self, camel_case: bool = False) -> dict[str, Any]:
+    def _prepare_asset_ids(self, resource: str, user_kwargs: dict[str, Any]) -> list[int]:
+        if self.id is None:
+            raise ValueError(f"Unable to fetch related {resource}, asset is missing id")
+        return remove_duplicates_keep_order([self.id, *user_kwargs.pop("asset_ids", [])])
+
+    def dump(self, camel_case: bool = True) -> dict[str, Any]:
         result = super().dump(camel_case)
-        if self.labels is not None:
-            result["labels"] = [label.dump(camel_case) for label in self.labels]
+        if isinstance(self.aggregates, AggregateResultItem):
+            result["aggregates"] = self.aggregates.dump(camel_case)
         return result
 
-    def to_pandas(
+    def to_pandas(  # type: ignore [override]
         self,
-        expand: Sequence[str] = ("metadata", "aggregates"),
+        expand_metadata: bool = False,
+        metadata_prefix: str = "metadata.",
+        expand_aggregates: bool = False,
+        aggregates_prefix: str = "aggregates.",
         ignore: list[str] | None = None,
         camel_case: bool = False,
+        convert_timestamps: bool = True,
     ) -> pandas.DataFrame:
         """Convert the instance into a pandas DataFrame.
 
         Args:
-            expand (Sequence[str]): List of row keys to expand, only works if the value is a Dict.
-            ignore (list[str] | None): List of row keys to not include when converting to a data frame.
-            camel_case (bool): Convert column names to camel case (e.g. `externalId` instead of `external_id`)
+            expand_metadata (bool): Expand the metadata into separate rows (default: False).
+            metadata_prefix (str): Prefix to use for the metadata rows, if expanded.
+            expand_aggregates (bool): Expand the aggregates into separate rows (default: False).
+            aggregates_prefix (str): Prefix to use for the aggregates rows, if expanded.
+            ignore (list[str] | None): List of row keys to skip when converting to a data frame. Is applied before expansions.
+            camel_case (bool): Convert attribute names to camel case (e.g. `externalId` instead of `external_id`). Does not affect custom data like metadata if expanded.
+            convert_timestamps (bool): Convert known attributes storing CDF timestamps (milliseconds since epoch) to datetime. Does not affect custom data like metadata.
 
         Returns:
             pandas.DataFrame: The dataframe.
         """
-        return super().to_pandas(expand=expand, ignore=ignore, camel_case=camel_case)
+        df = super().to_pandas(
+            expand_metadata=expand_metadata,
+            metadata_prefix=metadata_prefix,
+            ignore=ignore,
+            camel_case=camel_case,
+            convert_timestamps=convert_timestamps,
+        )
+        if not (expand_aggregates and "aggregates" in df.index):
+            return df
+
+        pd = local_import("pandas")
+        col = df.squeeze()
+        aggregates = convert_dict_to_case(col.pop("aggregates"), camel_case)
+        return pd.concat((col, pd.Series(aggregates).add_prefix(aggregates_prefix))).to_frame(name="value")
+
+
+class AssetWrite(AssetCore):
+    """A representation of a physical asset, for example, a factory or a piece of equipment. This is the
+    writing version of the Asset class, and is used when inserting new assets.
+
+    Args:
+        name (str): The name of the asset.
+        external_id (str | None): The external ID provided by the client. Must be unique for the resource type.
+        parent_id (int | None): The parent of the node, null if it is the root node.
+        parent_external_id (str | None): The external ID of the parent. The property is omitted if the asset doesn't have a parent or if the parent doesn't have externalId.
+        description (str | None): The description of the asset.
+        data_set_id (int | None): The id of the dataset this asset belongs to.
+        metadata (dict[str, str] | None): Custom, application-specific metadata. String key -> String value. Limits: Maximum length of key is 128 bytes, value 10240 bytes, up to 256 key-value pairs, of total size at most 10240.
+        source (str | None): The source of the asset.
+        labels (list[Label | str | LabelDefinitionWrite | dict] | None): A list of the labels associated with this resource item.
+        geo_location (GeoLocation | None): The geographic metadata of the asset.
+    """
+
+    def __init__(
+        self,
+        name: str,
+        external_id: str | None = None,
+        parent_id: int | None = None,
+        parent_external_id: str | None = None,
+        description: str | None = None,
+        data_set_id: int | None = None,
+        metadata: dict[str, str] | None = None,
+        source: str | None = None,
+        labels: list[Label | str | LabelDefinitionWrite | dict] | None = None,
+        geo_location: GeoLocation | None = None,
+    ) -> None:
+        super().__init__(
+            external_id=external_id,
+            name=name,
+            parent_id=parent_id,
+            parent_external_id=parent_external_id,
+            description=description,
+            data_set_id=data_set_id,
+            metadata=metadata,
+            source=source,
+            labels=Label._load_list(labels),
+            geo_location=geo_location,
+        )
+
+    @classmethod
+    def _load(cls, resource: dict, cognite_client: CogniteClient | None = None) -> AssetWrite:
+        return cls(
+            external_id=resource.get("externalId"),
+            name=resource["name"],
+            parent_id=resource.get("parentId"),
+            parent_external_id=resource.get("parentExternalId"),
+            description=resource.get("description"),
+            data_set_id=resource.get("dataSetId"),
+            metadata=resource.get("metadata"),
+            source=resource.get("source"),
+            labels=(labels := resource.get("labels")) and Label._load_list(labels),  # type: ignore[arg-type]
+            geo_location=(geo_location := resource.get("geoLocation")) and GeoLocation._load(geo_location),
+        )
+
+    def as_write(self) -> AssetWrite:
+        """Returns self."""
+        return self
 
 
 class AssetUpdate(CogniteUpdate):
@@ -355,83 +511,100 @@ class AssetUpdate(CogniteUpdate):
         return AssetUpdate._PrimitiveAssetUpdate(self, "geoLocation")
 
     @classmethod
-    def _get_update_properties(cls) -> list[PropertySpec]:
+    def _get_update_properties(cls, item: CogniteResource | None = None) -> list[PropertySpec]:
         return [
             # External ID is nullable, but is used in the upsert logic and thus cannot be nulled out.
             PropertySpec("external_id", is_nullable=False),
             PropertySpec("name", is_nullable=False),
             PropertySpec("description"),
             PropertySpec("data_set_id"),
-            PropertySpec("metadata", is_container=True),
+            PropertySpec("metadata", is_object=True),
             PropertySpec("source"),
             PropertySpec("parent_id", is_nullable=False),
             PropertySpec("parent_external_id", is_nullable=False),
-            PropertySpec("labels", is_container=True),
+            PropertySpec("labels", is_list=True),
             PropertySpec("geo_location"),
         ]
 
 
-class AssetList(CogniteResourceList[Asset], IdTransformerMixin):
+class AssetWriteList(CogniteResourceList[AssetWrite], ExternalIDTransformerMixin):
+    _RESOURCE = AssetWrite
+
+
+class AssetList(WriteableCogniteResourceList[AssetWrite, Asset], IdTransformerMixin):
     _RESOURCE = Asset
 
-    def __init__(self, resources: Collection[Any], cognite_client: CogniteClient | None = None) -> None:
-        super().__init__(resources, cognite_client)
-        self._retrieve_chunk_size = 100
+    def as_write(self) -> AssetWriteList:
+        return AssetWriteList([a.as_write() for a in self.data], cognite_client=self._get_cognite_client())
 
-    def time_series(self) -> TimeSeriesList:
+    def time_series(self, **kwargs: Any) -> TimeSeriesList:
         """Retrieve all time series related to these assets.
 
+        Args:
+            **kwargs (Any): All extra keyword arguments are passed to time_series/list. Note: 'partitions' and 'limit' can not be used.
         Returns:
             TimeSeriesList: All time series related to the assets in this AssetList.
         """
         from cognite.client.data_classes import TimeSeriesList
 
-        return self._retrieve_related_resources(TimeSeriesList, self._cognite_client.time_series)
+        return self._retrieve_related_resources(TimeSeriesList, self._cognite_client.time_series, kwargs)
 
-    def sequences(self) -> SequenceList:
+    def sequences(self, **kwargs: Any) -> SequenceList:
         """Retrieve all sequences related to these assets.
 
+        Args:
+            **kwargs (Any): All extra keyword arguments are passed to sequences/list. Note: 'limit' can not be used.
         Returns:
             SequenceList: All sequences related to the assets in this AssetList.
         """
         from cognite.client.data_classes import SequenceList
 
-        return self._retrieve_related_resources(SequenceList, self._cognite_client.sequences)
+        return self._retrieve_related_resources(SequenceList, self._cognite_client.sequences, kwargs)
 
-    def events(self) -> EventList:
+    def events(self, **kwargs: Any) -> EventList:
         """Retrieve all events related to these assets.
 
+        Args:
+            **kwargs (Any): All extra keyword arguments are passed to events/list. Note: 'sort', 'partitions' and 'limit' can not be used.
         Returns:
             EventList: All events related to the assets in this AssetList.
         """
         from cognite.client.data_classes import EventList
 
-        return self._retrieve_related_resources(EventList, self._cognite_client.events)
+        return self._retrieve_related_resources(EventList, self._cognite_client.events, kwargs, chunk_size=5000)
 
-    def files(self) -> FileMetadataList:
+    def files(self, **kwargs: Any) -> FileMetadataList:
         """Retrieve all files metadata related to these assets.
 
+        Args:
+            **kwargs (Any): All extra keyword arguments are passed to files/list. Note: 'limit' can not be used.
         Returns:
             FileMetadataList: Metadata about all files related to the assets in this AssetList.
         """
         from cognite.client.data_classes import FileMetadataList
 
-        return self._retrieve_related_resources(FileMetadataList, self._cognite_client.files)
+        return self._retrieve_related_resources(FileMetadataList, self._cognite_client.files, kwargs)
 
     def _retrieve_related_resources(
-        self, resource_list_class: type[T_CogniteResourceList], resource_api: Any
+        self,
+        resource_list_class: type[T_CogniteResourceList],
+        resource_api: Any,
+        user_kwargs: dict[str, Any],
+        chunk_size: int = 100,
     ) -> T_CogniteResourceList:
         seen: set[int] = set()
         add_to_seen = seen.add
         lock = threading.Lock()
 
+        ids = remove_duplicates_keep_order([a.id for a in self.data] + user_kwargs.pop("asset_ids", []))
+        user_kwargs.pop("sort", None), user_kwargs.pop("partitions", None), user_kwargs.pop("limit", None)
+
         def retrieve_and_deduplicate(asset_ids: list[int]) -> list[T_CogniteResource]:
-            res = resource_api.list(asset_ids=asset_ids, limit=-1)
+            res = resource_api.list(asset_ids=asset_ids, **user_kwargs, limit=None)
             with lock:
                 return [r for r in res if not (r.id in seen or add_to_seen(r.id))]
 
-        ids = [a.id for a in self.data]
-        tasks = [{"asset_ids": chunk} for chunk in split_into_chunks(ids, self._retrieve_chunk_size)]
+        tasks = [{"asset_ids": chunk} for chunk in split_into_chunks(set(ids), chunk_size)]
         res_list = execute_tasks(retrieve_and_deduplicate, tasks, resource_api._config.max_workers).results
         return resource_list_class(list(itertools.chain.from_iterable(res_list)), cognite_client=self._cognite_client)
 
@@ -442,8 +615,8 @@ class AssetFilter(CogniteFilter):
     Args:
         name (str | None): The name of the asset.
         parent_ids (Sequence[int] | None): Return only the direct descendants of the specified assets.
-        parent_external_ids (Sequence[str] | None): Return only the direct descendants of the specified assets.
-        asset_subtree_ids (Sequence[dict[str, Any]] | None): Only include assets in subtrees rooted at the specified assets (including the roots given). If the total size of the given subtrees exceeds 100,000 assets, an error will be returned.
+        parent_external_ids (SequenceNotStr[str] | None): Return only the direct descendants of the specified assets.
+        asset_subtree_ids (Sequence[dict[str, Any]] | None): Only include assets in subtrees rooted at the specified asset IDs and external IDs. If the total size of the given subtrees exceeds 100,000 assets, an error will be returned.
         data_set_ids (Sequence[dict[str, Any]] | None): No description.
         metadata (dict[str, str] | None): Custom, application specific metadata. String key -> String value. Limits: Maximum length of key is 128 bytes, value 10240 bytes, up to 256 key-value pairs, of total size at most 10240.
         source (str | None): The source of the asset.
@@ -459,7 +632,7 @@ class AssetFilter(CogniteFilter):
         self,
         name: str | None = None,
         parent_ids: Sequence[int] | None = None,
-        parent_external_ids: Sequence[str] | None = None,
+        parent_external_ids: SequenceNotStr[str] | None = None,
         asset_subtree_ids: Sequence[dict[str, Any]] | None = None,
         data_set_ids: Sequence[dict[str, Any]] | None = None,
         metadata: dict[str, str] | None = None,
@@ -488,10 +661,17 @@ class AssetFilter(CogniteFilter):
         if labels is not None and not isinstance(labels, LabelFilter):
             raise TypeError("AssetFilter.labels must be of type LabelFilter")
 
-    def dump(self, camel_case: bool = False) -> dict[str, Any]:
+    def dump(self, camel_case: bool = True) -> dict[str, Any]:
         result = super().dump(camel_case)
         if isinstance(self.labels, LabelFilter):
             result["labels"] = self.labels.dump(camel_case)
+        if isinstance(self.geo_location, GeoLocationFilter):
+            result["geoLocation" if camel_case else "geo_location"] = self.geo_location.dump(camel_case)
+        if isinstance(self.created_time, TimestampRange):
+            result["createdTime" if camel_case else "created_time"] = self.created_time.dump(camel_case)
+        if isinstance(self.last_updated_time, TimestampRange):
+            result["lastUpdatedTime" if camel_case else "last_updated_time"] = self.last_updated_time.dump(camel_case)
+
         return result
 
 
@@ -501,7 +681,7 @@ class AssetHierarchy:
     any asset providing a parent link by ID instead of external ID, are assumed valid.
 
     Args:
-        assets (Sequence[Asset]): Sequence of assets to be inspected for validity.
+        assets (Sequence[Asset | AssetWrite]): Sequence of assets to be inspected for validity.
         ignore_orphans (bool): If true, orphan assets are assumed valid and won't raise.
 
     Examples:
@@ -539,8 +719,8 @@ class AssetHierarchy:
         ...     report = file_like.getvalue()
     """
 
-    def __init__(self, assets: Sequence[Asset], ignore_orphans: bool = False) -> None:
-        self._assets = assets
+    def __init__(self, assets: Sequence[Asset | AssetWrite], ignore_orphans: bool = False) -> None:
+        self._assets = self._convert_to_read(assets)
         self._roots: list[Asset] | None = None
         self._orphans: list[Asset] | None = None
         self._ignore_orphans = ignore_orphans
@@ -554,6 +734,14 @@ class AssetHierarchy:
     def __len__(self) -> int:
         return len(self._assets)
 
+    @staticmethod
+    def _convert_to_read(assets: Sequence[Asset | AssetWrite]) -> Sequence[Asset]:
+        # TODO: AssetHierarchy doesn't work with AssetWrite (or more correctly, _AssetHierarchyCreator...)
+        #       and as we don't have a reverse of "as_write", we dump-load the write into a read:
+        return [
+            Asset._load(asset.dump(camel_case=True)) if isinstance(asset, AssetWrite) else asset for asset in assets
+        ]
+
     def is_valid(self, on_error: Literal["ignore", "warn", "raise"] = "ignore") -> bool:
         if not self.__validation_has_run:
             self.validate(verbose=False, on_error="ignore")
@@ -697,7 +885,7 @@ class AssetHierarchy:
         counts.sort(key=lambda args: -args[-1])
         # The count for the fictitious "root of roots" is just len(assets), so we remove it:
         (count_dct := dict(counts)).pop(None, None)
-        return count_dct
+        return cast(dict[str, int], count_dct)
 
     def _on_error(self, on_error: Literal["ignore", "warn", "raise"], message: str) -> None:
         if on_error == "warn":
@@ -711,7 +899,8 @@ class AssetHierarchy:
         )
 
     def _inspect_attributes(self) -> tuple[list[Asset], list[Asset], list[Asset], list[Asset], dict[str, list[Asset]]]:
-        invalid, orphans, roots, unsure_parents, duplicates = [], [], [], [], defaultdict(list)
+        invalid, orphans, roots, unsure_parents = [], [], [], []  # type: tuple[list[Asset], list[Asset], list[Asset], list[Asset]]
+        duplicates: defaultdict[str, list[Asset]] = defaultdict(list)
         xid_count = Counter(a.external_id for a in self._assets)
 
         for asset in self._assets:
@@ -749,10 +938,10 @@ class AssetHierarchy:
     def _locate_cycles(self) -> tuple[int, list[list[str]]]:
         has_cycles = set()
         no_cycles = {None, *(a.external_id for a in self._roots or [])}
-        edges = cast(Dict[str, Optional[str]], {a.external_id: a.parent_external_id for a in self._assets})
+        edges = cast(dict[str, str | None], {a.external_id: a.parent_external_id for a in self._assets})
 
         if self._ignore_orphans:
-            no_cycles |= {a.parent_external_id for a in cast(List[Asset], self._orphans)}
+            no_cycles |= {a.parent_external_id for a in cast(list[Asset], self._orphans)}
 
         for xid, parent in edges.items():
             if parent in no_cycles:
@@ -874,35 +1063,35 @@ class AssetHierarchy:
 
 
 class AssetProperty(EnumProperty):
-    labels = "labels"
-    created_time = "createdTime"
-    data_set_id = "dataSetId"
-    id = "id"
-    last_updated_time = "lastUpdatedTime"
-    parent_id = "parentId"
-    root_id = "rootId"
-    description = "description"
-    external_id = "externalId"
-    metadata = "metadata"
-    name = "name"
-    source = "source"
+    labels = auto()
+    created_time = auto()
+    data_set_id = auto()
+    id = auto()
+    last_updated_time = auto()
+    parent_id = auto()
+    root_id = auto()
+    description = auto()
+    external_id = auto()
+    metadata = auto()
+    name = auto()  # type: ignore [assignment]
+    source = auto()
 
     @staticmethod
     def metadata_key(key: str) -> list[str]:
         return ["metadata", key]
 
 
-AssetPropertyLike: TypeAlias = Union[AssetProperty, str, List[str]]
+AssetPropertyLike: TypeAlias = AssetProperty | str | list[str]
 
 
 class SortableAssetProperty(EnumProperty):
-    created_time = "createdTime"
-    data_set_id = "dataSetId"
-    description = "description"
-    external_id = "externalId"
-    last_updated_time = "lastUpdatedTime"
-    name = "name"
-    source = "source"
+    created_time = auto()
+    data_set_id = auto()
+    description = auto()
+    external_id = auto()
+    last_updated_time = auto()
+    name = auto()  # type: ignore [assignment]
+    source = auto()
     score = "_score_"
 
     @staticmethod
@@ -910,7 +1099,7 @@ class SortableAssetProperty(EnumProperty):
         return ["metadata", key]
 
 
-SortableAssetPropertyLike: TypeAlias = Union[SortableAssetProperty, str, List[str]]
+SortableAssetPropertyLike: TypeAlias = SortableAssetProperty | str | list[str]
 
 
 class AssetSort(CogniteSort):
