@@ -200,7 +200,7 @@ class Capability(ABC):
                     allow_unknown=allow_unknown,
                 ),
             )
-        # We infer this as an unknown acl not yet added to the SDK. All API-responses are neccessarily
+        # We infer this as an unknown acl not yet added to the SDK. All API-responses are necessarily
         # loaded with 'allow_unknown=True' (to future-proof):
         if allow_unknown:
             if len(resource) != 1:
@@ -344,10 +344,8 @@ class ProjectCapabilityList(CogniteResourceList[ProjectCapability]):
             if isinstance(cap, LegacyCapability):
                 # Legacy capabilities are no longer in use, so they are safe to skip.
                 continue
-            if (
-                isinstance(proj_cap.project_scope, AllProjectsScope)
-                or isinstance(proj_cap.project_scope, ProjectsScope)
-                and project in proj_cap.project_scope.projects
+            if isinstance(proj_cap.project_scope, AllProjectsScope) or (
+                isinstance(proj_cap.project_scope, ProjectsScope) and project in proj_cap.project_scope.projects
             ):
                 output |= cap.as_tuples()
         return output
@@ -389,6 +387,15 @@ class IDScopeLowerCase(Capability.Scope):
         return {(self._scope_name, i) for i in self.ids}
 
 
+@dataclass(frozen=True)
+class InstancesScope(Capability.Scope):
+    _scope_name = "instancesScope"
+    instances: list[str]
+
+    def as_tuples(self) -> set[tuple[str, str]]:
+        return {(self._scope_name, s) for s in self.instances}
+
+
 @dataclass(frozen=True)
 class ExtractionPipelineScope(Capability.Scope):
     _scope_name = "extractionPipelineScope"
@@ -401,6 +408,18 @@ class ExtractionPipelineScope(Capability.Scope):
         return {(self._scope_name, i) for i in self.ids}
 
 
+@dataclass(frozen=True)
+class PostgresGatewayUsersScope(Capability.Scope):
+    _scope_name = "usersScope"
+    usernames: list[str]
+
+    def __post_init__(self) -> None:
+        object.__setattr__(self, "usernames", [str(i) for i in self.usernames])
+
+    def as_tuples(self) -> set[tuple[str, str]]:
+        return {(self._scope_name, i) for i in self.usernames}
+
+
 @dataclass(frozen=True)
 class DataSetScope(Capability.Scope):
     _scope_name = "datasetScope"
@@ -469,6 +488,15 @@ class SpaceIDScope(Capability.Scope):
         return {(self._scope_name, s) for s in self.space_ids}
 
 
+@dataclass(frozen=True)
+class AppConfigScope(Capability.Scope):
+    _scope_name = "appScope"
+    apps: list[Literal["SEARCH"]]
+
+    def as_tuples(self) -> set[tuple[str, str]]:
+        return {(self._scope_name, i) for i in self.apps}
+
+
 @dataclass(frozen=True)
 class PartitionScope(Capability.Scope):
     _scope_name = "partition"
@@ -536,6 +564,21 @@ class UnknownAcl(Capability):
         return ""
 
 
+@dataclass
+class AgentsAcl(Capability):
+    _capability_name = "agentsAcl"
+    actions: Sequence[Action]
+    scope: AllScope
+
+    class Action(Capability.Action):  # type: ignore [misc]
+        READ = "READ"
+        WRITE = "WRITE"
+        RUN = "RUN"
+
+    class Scope:
+        All = AllScope
+
+
 @dataclass
 class AnalyticsAcl(Capability):
     _capability_name = "analyticsAcl"
@@ -567,6 +610,21 @@ class AnnotationsAcl(Capability):
         All = AllScope
 
 
+@dataclass
+class AppConfigAcl(Capability):
+    _capability_name = "appConfigAcl"
+    actions: Sequence[Action]
+    scope: AllScope | AppConfigScope
+
+    class Action(Capability.Action):  # type: ignore [misc]
+        Read = "READ"
+        Write = "WRITE"
+
+    class Scope:
+        All = AllScope
+        AppConfig = AppConfigScope
+
+
 @dataclass
 class AssetsAcl(Capability):
     _capability_name = "assetsAcl"
@@ -598,6 +656,20 @@ class DataSetsAcl(Capability):
         ID = IDScope
 
 
+@dataclass
+class DiagramParsingAcl(Capability):
+    _capability_name = "diagramParsingAcl"
+    actions: Sequence[Action]
+    scope: AllScope = field(default_factory=AllScope)
+
+    class Action(Capability.Action):  # type: ignore [misc]
+        Read = "READ"
+        Write = "WRITE"
+
+    class Scope:
+        All = AllScope
+
+
 @dataclass
 class DigitalTwinAcl(Capability):
     _capability_name = "digitalTwinAcl"
@@ -859,6 +931,36 @@ class RoboticsAcl(Capability):
         DataSet = DataSetScope
 
 
+@dataclass
+class SAPWritebackAcl(Capability):
+    _capability_name = "sapWritebackAcl"
+    actions: Sequence[Action]
+    scope: AllScope | InstancesScope
+
+    class Action(Capability.Action):  # type: ignore [misc]
+        Read = "READ"
+        Write = "WRITE"
+
+    class Scope:
+        All = AllScope
+        Instances = InstancesScope
+
+
+@dataclass
+class SAPWritebackRequestsAcl(Capability):
+    _capability_name = "sapWritebackRequestsAcl"
+    actions: Sequence[Action]
+    scope: AllScope | InstancesScope
+
+    class Action(Capability.Action):  # type: ignore [misc]
+        Write = "WRITE"
+        List = "LIST"
+
+    class Scope:
+        All = AllScope
+        Instances = InstancesScope
+
+
 @dataclass
 class SecurityCategoriesAcl(Capability):
     _capability_name = "securityCategoriesAcl"
@@ -1236,13 +1338,14 @@ class WorkflowOrchestrationAcl(Capability):
 
     class Scope:
         All = AllScope
+        DataSet = DataSetScope
 
 
 @dataclass
 class PostgresGatewayAcl(Capability):
     _capability_name = "postgresGatewayAcl"
     actions: Sequence[Action]
-    scope: AllScope = field(default_factory=AllScope)
+    scope: AllScope | PostgresGatewayUsersScope = field(default_factory=AllScope)
 
     class Action(Capability.Action):  # type: ignore [misc]
         Read = "READ"
@@ -1250,6 +1353,7 @@ class PostgresGatewayAcl(Capability):
 
     class Scope:
         All = AllScope
+        UsersScope = PostgresGatewayUsersScope
 
 
 @dataclass
