@@ -17,6 +17,7 @@ from cognite.client.data_classes._base import (
     CogniteResource,
     CogniteResourceList,
     CogniteUpdate,
+    IdTransformerMixin,
     PropertySpec,
 )
 from cognite.client.data_classes.annotation_types.images import (
@@ -39,7 +40,7 @@ if TYPE_CHECKING:
 
 
 class JobStatus(Enum):
-    _NOT_STARTED: None = None
+    _NOT_STARTED = None
     QUEUED = "Queued"
     RUNNING = "Running"
     COMPLETED = "Completed"
@@ -349,7 +350,7 @@ class EntityMatchingModelUpdate(CogniteUpdate):
         ]
 
 
-class EntityMatchingModelList(CogniteResourceList[EntityMatchingModel]):
+class EntityMatchingModelList(CogniteResourceList[EntityMatchingModel], IdTransformerMixin):
     _RESOURCE = EntityMatchingModel
 
 
@@ -1042,10 +1043,10 @@ class ResourceReference(CogniteResource):
     ) -> None:
         self.id = id
         self.external_id = external_id
-        self._cognite_client: CogniteClient = cast("CogniteClient", None)  # Read only
+        self._cognite_client = None  # Read only
 
 
-class ResourceReferenceList(CogniteResourceList[ResourceReference]):
+class ResourceReferenceList(CogniteResourceList[ResourceReference], IdTransformerMixin):
     _RESOURCE = ResourceReference
 
 
