@@ -7,16 +7,17 @@ from typing import TYPE_CHECKING, Any, Literal, NoReturn, TypeAlias, cast, final
 
 from cognite.client.data_classes._base import EnumProperty, Geometry
 from cognite.client.data_classes.labels import Label
+from cognite.client.utils._identifier import InstanceId
 from cognite.client.utils._text import convert_all_keys_to_camel_case, to_camel_case
-from cognite.client.utils.useful_types import SequenceNotStr
+from cognite.client.utils.useful_types import SequenceNotStr, is_sequence_not_str
 
 if TYPE_CHECKING:
     from cognite.client.data_classes.data_modeling.ids import ContainerId, ViewId
 
 
-PropertyReference: TypeAlias = str | tuple[str, ...] | list[str] | EnumProperty
+PropertyReference: TypeAlias = str | SequenceNotStr[str] | EnumProperty
 
-RawValue: TypeAlias = str | float | bool | Sequence | Mapping[str, Any] | Label
+RawValue: TypeAlias = str | float | bool | Sequence | Mapping[str, Any] | Label | InstanceId
 
 
 @dataclass
@@ -42,6 +43,9 @@ def _dump_filter_value(value: FilterValueList | FilterValue) -> Any:
     elif isinstance(value, ParameterValue):
         return {"parameter": value.parameter}
 
+    elif isinstance(value, InstanceId):
+        return value.dump(include_instance_type=False)
+
     elif hasattr(value, "dump"):
         return value.dump()
 
@@ -188,6 +192,8 @@ class Filter(ABC):
                 previously_referenced_properties=filter_body["previouslyReferencedProperties"],
                 filter_type=filter_body["filterType"],
             )
+        elif (filter_body := filter_.get(InstanceReferences._filter_name)) is not None:
+            return InstanceReferences(references=filter_body)
         else:
             filter_name, filter_body = next(iter(filter_.items()))
             return UnknownFilter(filter_name, filter_body)
@@ -202,6 +208,21 @@ class Filter(ABC):
                 output.update(filter_._involved_filter_types())
         return output
 
+    def _list_filters_without_nesting(self, other: Filter, operator: type[CompoundFilter]) -> list[Filter]:
+        filters: list[Filter] = []
+        filters.extend(self._filters) if isinstance(self, operator) else filters.append(self)
+        filters.extend(other._filters) if isinstance(other, operator) else filters.append(other)
+        return filters
+
+    def __and__(self, other: Filter) -> And:
+        return And(*self._list_filters_without_nesting(other, And))
+
+    def __or__(self, other: Filter) -> Or:
+        return Or(*self._list_filters_without_nesting(other, Or))
+
+    def __invert__(self) -> Not:
+        return Not(self)
+
 
 class UnknownFilter(Filter):
     def __init__(self, filter_name: str, filter_body: dict[str, Any]) -> None:
@@ -258,6 +279,8 @@ class CompoundFilter(Filter, ABC):
     def __init__(self, *filters: Filter) -> None:
         if not_flt := [flt for flt in filters if not isinstance(flt, Filter)]:
             raise TypeError(f"One or more invalid filters, expected Filter, got: {not_flt}")
+        if not filters:
+            raise TypeError("At least one filter must be provided")
         self._filters = filters
 
     def _filter_body(self, camel_case_property: bool) -> list | dict:
@@ -322,6 +345,10 @@ class And(CompoundFilter):
             >>> flt = And(
             ...     Equals(my_view.as_property_ref("some_property"), 42),
             ...     In(my_view.as_property_ref("another_property"), ["a", "b", "c"]))
+
+        Using the "&" operator:
+
+            >>> flt = Equals("age", 42) & Equals("name", "Alice")
     """
 
     _filter_name = "and"
@@ -349,6 +376,10 @@ class Or(CompoundFilter):
             >>> flt = Or(
             ...     Equals(my_view.as_property_ref("some_property"), 42),
             ...     In(my_view.as_property_ref("another_property"), ["a", "b", "c"]))
+
+        Using the "|" operator:
+
+            >>> flt = Equals("name", "Bob") | Equals("name", "Alice")
     """
 
     _filter_name = "or"
@@ -374,10 +405,17 @@ class Not(CompoundFilter):
 
             >>> is_42 = Equals(my_view.as_property_ref("some_property"), 42)
             >>> flt = Not(is_42)
+
+        Using the "~" operator:
+
+            >>> flt = ~Equals("name", "Bob")
     """
 
     _filter_name = "not"
 
+    def __init__(self, filter: Filter) -> None:
+        super().__init__(filter)
+
     def _filter_body(self, camel_case_property: bool) -> dict:
         return self._filters[0].dump(camel_case_property)
 
@@ -859,7 +897,7 @@ class SpaceFilter(FilterWithProperty):
 
     @classmethod
     def load(cls, filter_: dict[str, Any]) -> NoReturn:
-        raise NotImplementedError("Custom filter 'SpaceFilter' can not be loaded")
+        raise NotImplementedError("Custom filter 'SpaceFilter' cannot be loaded")
 
     def _filter_body(self, camel_case_property: bool) -> dict[str, Any]:
         return {
@@ -869,3 +907,70 @@ class SpaceFilter(FilterWithProperty):
 
     def _involved_filter_types(self) -> set[type[Filter]]:
         return self._involved_filter
+
+
+class IsNull(Not):  # type: ignore [misc]
+    """Data modeling filter for instances whose property is null, effectively a negated Exists-filter.
+
+    Args:
+        property (SequenceNotStr[str]): The property to filter on.
+
+    Example:
+        Filter than can be used to retrieve instances where the property value is not set:
+
+        - A filter using a tuple to reference the property:
+
+            >>> from cognite.client.data_classes.filters import IsNull
+            >>> flt = IsNull(("space", "view_xid/version", "some_property"))
+
+        - Composing the property reference using the ``View.as_property_ref`` method:
+
+            >>> flt = IsNull(my_view.as_property_ref("some_property"))
+
+    """
+
+    _filter_name = "__is_null"
+
+    def __init__(self, property: SequenceNotStr[str]) -> None:
+        if not is_sequence_not_str(property):
+            raise TypeError(
+                "The IsNull filter is a Data Modeling filter and expected a sequence of str to describe the property, "
+                f"like ['node', 'space'] or ['my-space', 'my-view/version', 'my-property'], got: {property}"
+            )
+        super().__init__(Exists(property))
+        self._filter_name = Not._filter_name
+
+    @classmethod
+    def load(cls, filter_: dict[str, Any]) -> NoReturn:
+        raise NotImplementedError("Custom filter 'IsNull' cannot be loaded")
+
+    def _involved_filter_types(self) -> set[type[Filter]]:
+        return {Not, Exists}
+
+
+class InstanceReferences(Filter):
+    """Data modeling filter which matches instances with these fully qualified references.
+
+    Args:
+        references (Sequence[InstanceId] | Sequence[tuple[str, str]]): The instance references.
+
+    Example:
+        Filter than can be used to retrieve instances where their space/externalId matches any of the provided values:
+
+        - A filter using a tuple as instance reference:
+
+            >>> from cognite.client.data_classes.filters import InstanceReferences, InstanceId
+            >>> flt = InstanceReferences([("someSpace", "someExternalId")])
+
+        - Composing the instance references using the InstanceId class:
+
+            >>> flt = InstanceReferences([InstanceId("someSpace", "someExternalId")])
+    """
+
+    _filter_name = "instanceReferences"
+
+    def __init__(self, references: Sequence[InstanceId] | Sequence[tuple[str, str]]) -> None:
+        self.__references = [InstanceId.load(ref) for ref in references]
+
+    def _filter_body(self, camel_case_property: bool) -> list:
+        return [ref.dump(camel_case=True, include_instance_type=False) for ref in self.__references]
