@@ -1,21 +1,50 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Sequence, cast
+from abc import ABC
+from collections.abc import Sequence
+from typing import TYPE_CHECKING, Any, cast
 
 from cognite.client.data_classes._base import (
     CogniteFilter,
-    CognitePropertyClassUtil,
-    CogniteResource,
+    CogniteObject,
     CogniteResourceList,
+    ExternalIDTransformerMixin,
+    WriteableCogniteResource,
+    WriteableCogniteResourceList,
 )
-from cognite.client.utils._text import convert_all_keys_to_camel_case, to_camel_case
+from cognite.client.utils.useful_types import SequenceNotStr
 
 if TYPE_CHECKING:
     from cognite.client import CogniteClient
 
 
-class LabelDefinition(CogniteResource):
+class LabelDefinitionCore(WriteableCogniteResource["LabelDefinitionWrite"], ABC):
     """A label definition is a globally defined label that can later be attached to resources (e.g., assets). For example, can you define a "Pump" label definition and attach that label to your pump assets.
+    This is the parent for the reading and writing versions.
+
+    Args:
+        external_id (str | None): The external ID provided by the client. Must be unique for the resource type.
+        name (str | None): Name of the label.
+        description (str | None): Description of the label.
+        data_set_id (int | None): The id of the dataset this label belongs to.
+    """
+
+    def __init__(
+        self,
+        external_id: str | None = None,
+        name: str | None = None,
+        description: str | None = None,
+        data_set_id: int | None = None,
+    ) -> None:
+        self.external_id = external_id
+        self.name = name
+        self.description = description
+        self.data_set_id = data_set_id
+
+
+class LabelDefinition(LabelDefinitionCore):
+    """A label definition is a globally defined label that can later be attached to resources (e.g., assets). For example, can you define a "Pump" label definition and attach that label to your pump assets.
+    This is the reading version of the LabelDefinition class. It is used when retrieving existing label definitions.
 
     Args:
         external_id (str | None): The external ID provided by the client. Must be unique for the resource type.
@@ -35,13 +64,65 @@ class LabelDefinition(CogniteResource):
         data_set_id: int | None = None,
         cognite_client: CogniteClient | None = None,
     ) -> None:
-        self.external_id = external_id
-        self.name = name
-        self.description = description
+        super().__init__(
+            external_id=external_id,
+            name=name,
+            description=description,
+            data_set_id=data_set_id,
+        )
         self.created_time = created_time
-        self.data_set_id = data_set_id
         self._cognite_client = cast("CogniteClient", cognite_client)
 
+    def as_write(self) -> LabelDefinitionWrite:
+        """Returns this LabelDefinition in its writing version."""
+        if self.external_id is None or self.name is None:
+            raise ValueError("External ID and name are required for the writing version of a label definition.")
+        return LabelDefinitionWrite(
+            external_id=self.external_id,
+            name=self.name,
+            description=self.description,
+            data_set_id=self.data_set_id,
+        )
+
+
+class LabelDefinitionWrite(LabelDefinitionCore):
+    """A label definition is a globally defined label that can later be attached to resources (e.g., assets). For example, can you define a "Pump" label definition and attach that label to your pump assets.
+    This is the writing version of the LabelDefinition class. It is used when creating new label definitions.
+
+    Args:
+        external_id (str): The external ID provided by the client. Must be unique for the resource type.
+        name (str): Name of the label.
+        description (str | None): Description of the label.
+        data_set_id (int | None): The id of the dataset this label belongs to.
+    """
+
+    def __init__(
+        self,
+        external_id: str,
+        name: str,
+        description: str | None = None,
+        data_set_id: int | None = None,
+    ) -> None:
+        super().__init__(
+            external_id=external_id,
+            name=name,
+            description=description,
+            data_set_id=data_set_id,
+        )
+
+    @classmethod
+    def _load(cls, resource: dict[str, Any], cognite_client: CogniteClient | None = None) -> LabelDefinitionWrite:
+        return cls(
+            external_id=resource["externalId"],
+            name=resource["name"],
+            description=resource.get("description"),
+            data_set_id=resource.get("dataSetId"),
+        )
+
+    def as_write(self) -> LabelDefinitionWrite:
+        """Returns this LabelDefinitionWrite instance."""
+        return self
+
 
 class LabelDefinitionFilter(CogniteFilter):
     """Filter on labels definitions with strict matching.
@@ -66,51 +147,60 @@ class LabelDefinitionFilter(CogniteFilter):
         self._cognite_client = cast("CogniteClient", cognite_client)
 
 
-class LabelDefinitionList(CogniteResourceList[LabelDefinition]):
+class LabelDefinitionWriteList(CogniteResourceList[LabelDefinitionWrite], ExternalIDTransformerMixin):
+    _RESOURCE = LabelDefinitionWrite
+
+
+class LabelDefinitionList(
+    WriteableCogniteResourceList[LabelDefinitionWrite, LabelDefinition], ExternalIDTransformerMixin
+):
     _RESOURCE = LabelDefinition
 
+    def as_write(self) -> LabelDefinitionWriteList:
+        """Returns this LabelDefinitionList in its writing version."""
+        return LabelDefinitionWriteList(
+            [item.as_write() for item in self.data], cognite_client=self._get_cognite_client()
+        )
 
-class Label(dict):
+
+class Label(CogniteObject):
     """A label assigned to a resource.
 
     Args:
         external_id (str | None): The external id to the attached label.
-        **kwargs (Any): No description.
+        **_ (Any): No description.
     """
 
-    def __init__(self, external_id: str | None = None, **kwargs: Any) -> None:
+    def __init__(self, external_id: str | None = None, **_: Any) -> None:
         self.external_id = external_id
-        self.update(kwargs)
-
-    external_id = CognitePropertyClassUtil.declare_property("externalId")
 
     @classmethod
-    def _load_list(cls, labels: Sequence[str | dict | LabelDefinition | Label] | None) -> list[Label] | None:
-        def convert_label(label: Label | str | LabelDefinition | dict) -> Label:
+    def _load_list(
+        cls,
+        labels: SequenceNotStr[str | dict | LabelDefinitionCore | Label]
+        | Sequence[dict | LabelDefinitionCore | Label]
+        | None,
+    ) -> list[Label] | None:
+        def convert_label(label: Label | str | LabelDefinitionCore | dict) -> Label:
             if isinstance(label, Label):
                 return label
             elif isinstance(label, str):
                 return Label(label)
-            elif isinstance(label, LabelDefinition):
+            elif isinstance(label, LabelDefinitionCore):
                 return Label(label.external_id)
             elif isinstance(label, dict):
                 if "externalId" in label:
                     return Label(label["externalId"])
+                if "external_id" in label:
+                    return Label(label["external_id"])
             raise ValueError(f"Could not parse label: {label}")
 
         if labels is None:
             return None
         return [convert_label(label) for label in labels]
 
-    @classmethod
-    def _load(cls, raw_label: dict[str, Any]) -> Label:
-        return cls(external_id=raw_label["externalId"])
 
-    def dump(self, camel_case: bool = False) -> dict[str, Any]:
-        return convert_all_keys_to_camel_case(self) if camel_case else dict(self)
-
-
-class LabelFilter(dict, CogniteFilter):
+class LabelFilter(CogniteFilter):
     """Return only the resource matching the specified label constraints.
 
     Args:
@@ -141,15 +231,21 @@ class LabelFilter(dict, CogniteFilter):
         self.contains_all = contains_all
         self._cognite_client = cast("CogniteClient", cognite_client)
 
-    @staticmethod
-    def _wrap_labels(values: list[str] | None) -> list[dict[str, str]] | None:
-        if values is None:
-            return None
-        return [{"externalId": v} for v in values]
+    @classmethod
+    def _load(cls, label_filter: dict[str, Any]) -> LabelFilter:
+        return cls(
+            contains_any=(any_ := label_filter.get("containsAny")) and [item["externalId"] for item in any_],
+            contains_all=(all_ := label_filter.get("containsAll")) and [item["externalId"] for item in all_],
+        )
 
-    def dump(self, camel_case: bool = False) -> dict[str, Any]:
-        keys = map(to_camel_case, self.keys()) if camel_case else self.keys()
-        return dict(zip(keys, map(self._wrap_labels, self.values())))
-
-    contains_any = CognitePropertyClassUtil.declare_property("containsAny")
-    contains_all = CognitePropertyClassUtil.declare_property("containsAll")
+    def dump(self, camel_case: bool = True) -> dict[str, Any]:
+        dumped: dict[str, Any] = {}
+        if self.contains_any:
+            dumped["containsAny"] = [
+                {"externalId" if camel_case else "external_id": item} for item in self.contains_any
+            ]
+        if self.contains_all:
+            dumped["containsAll"] = [
+                {"externalId" if camel_case else "external_id": item} for item in self.contains_all
+            ]
+        return dumped
