@@ -5,7 +5,6 @@ import math
 import platform
 import warnings
 from collections.abc import Hashable, Iterable, Iterator, Sequence
-from threading import Thread
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -16,13 +15,13 @@ from typing import (
 from urllib.parse import quote
 
 from cognite.client.utils import _json
+from cognite.client.utils._importing import local_import
 from cognite.client.utils._text import (
     convert_all_keys_to_camel_case,
     convert_all_keys_to_snake_case,
     to_camel_case,
     to_snake_case,
 )
-from cognite.client.utils._version_checker import get_newest_version_in_major_release
 from cognite.client.utils.useful_types import SequenceNotStr
 
 if TYPE_CHECKING:
@@ -42,6 +41,10 @@ def is_finite(limit: Any) -> TypeGuard[int]:
     return isinstance(limit, int) and limit >= 0
 
 
+def is_positive(limit: Any) -> TypeGuard[int]:
+    return isinstance(limit, int) and limit > 0
+
+
 def is_unlimited(limit: float | int | None) -> bool:
     return limit in {None, -1, math.inf}
 
@@ -83,8 +86,7 @@ def fast_dict_load(
 
 def load_yaml_or_json(resource: str) -> Any:
     try:
-        import yaml
-
+        yaml = local_import("yaml")
         return yaml.safe_load(resource)
     except ImportError:
         return _json.loads(resource)
@@ -154,24 +156,6 @@ def get_user_agent() -> str:
     return f"{sdk_version} {python_version} {operating_system}"
 
 
-# Wrap in a cache to ensure we only ever run the version check once.
-@functools.lru_cache(1)
-def _check_client_has_newest_major_version() -> None:
-    def run() -> None:
-        version = get_current_sdk_version()
-        newest_version = get_newest_version_in_major_release("cognite-sdk", version)
-        if newest_version != version:
-            warnings.warn(
-                f"You are using {version=} of the SDK, however version='{newest_version}' is available. "
-                "To suppress this warning, either upgrade or do the following:\n"
-                ">>> from cognite.client.config import global_config\n"
-                ">>> global_config.disable_pypi_version_check = True",
-                stacklevel=3,
-            )
-
-    Thread(target=run, daemon=True).start()
-
-
 @overload
 def split_into_n_parts(seq: list[T], *, n: int) -> Iterator[list[T]]: ...
 
@@ -206,6 +190,11 @@ def split_into_chunks(
         collection = list(collection.items())
         return [dict(collection[i : i + chunk_size]) for i in range(0, len(collection), chunk_size)]
 
+    from cognite.client.data_classes.datapoints import Datapoints, DatapointsArray
+
+    if isinstance(collection, (DatapointsArray, Datapoints)):
+        return [collection[i : i + chunk_size] for i in range(0, len(collection), chunk_size)]
+
     raise TypeError(f"Can only split list or dict, not {type(collection)}")
 
 
@@ -230,7 +219,7 @@ def find_duplicates(seq: Iterable[THashable]) -> set[THashable]:
     return {x for x in seq if x in seen or add(x)}
 
 
-def remove_duplicates_keep_order(seq: Sequence[THashable]) -> list[THashable]:
+def remove_duplicates_keep_order(seq: Iterable[THashable]) -> list[THashable]:
     seen: set[THashable] = set()
     add = seen.add
     return [x for x in seq if x not in seen and not add(x)]
